/*
* Copyright 2019-2020 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This file is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this file.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "../CBiosEDPPanel.h"
#include "../../../Hw/HwBlock/CBiosDIU_DP.h"

// eDP panel pre-initialization, if something must be done before detecting, do here, and set EDO156_Panel_Desc.EDPPreCaps.IsNeedPreInit = CBIOS_FALSE.
CBIOS_STATUS EDO156_PreInit(PCBIOS_VOID pvcbe)
{
    CBIOS_STATUS Status = CBIOS_ER_NOT_YET_IMPLEMENTED;

    return Status;
}

// eDP panel initialization
CBIOS_STATUS EDO156_Init(PCBIOS_VOID pvcbe)
{
    CBIOS_STATUS Status = CBIOS_ER_NOT_YET_IMPLEMENTED;

    return Status;
}

CBIOS_STATUS EDO156_DeInit(PCBIOS_VOID pvcbe)
{
    CBIOS_STATUS Status = CBIOS_ER_NOT_YET_IMPLEMENTED;

    return Status;
}

// get panel backlight
CBIOS_STATUS EDO156_GetBacklight(PCBIOS_VOID pvcbe, CBIOS_MODULE_INDEX DPModuleIndex, CBIOS_U32 *pBacklightValue)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_STATUS         Status = CBIOS_OK;
    CBIOS_U8                 Data[2] = {0};

    if(cbDIU_Aux_Dpcd_Read(pcbe, DPModuleIndex, 0x354, Data, 2))
    {
        *pBacklightValue = cbRound(((Data[1]<<8)|Data[0]) * 0xFF, 400, ROUND_UP);
    }
    else
    {
        Status = CBIOS_ER_INTERNAL;
    }

    return Status;
}

CBIOS_STATUS EDO156_SetBacklightUnLock(PCBIOS_VOID pvcbe, CBIOS_MODULE_INDEX DPModuleIndex, CBIOS_U32 BacklightValue)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_STATUS        Status = CBIOS_OK;
    CBIOS_U8                Data[2] = {0};

    BacklightValue = cbRound(BacklightValue * 400, 0xFF, ROUND_DOWN);

    //the panel set brightness less than 3, the brightness is like 400
    if(BacklightValue < 3)
    {
        BacklightValue = 3;
    }

    Data[0] = BacklightValue & 0xFF;
    Data[1] = (BacklightValue >> 8) & 0xFF;

    if(!cbDIU_Aux_Dpcd_Write(pcbe, DPModuleIndex, 0x354, Data, 2))
    {
        Status = CBIOS_ER_INTERNAL;
        cbDebugPrint((MAKE_LEVEL(DP, DEBUG), "%s: Write DPCD 0x354 fail!\n", FUNCTION_NAME));
    }

    return Status;
}


/* the EDO 156 DPCD and brightness table
DPCD 355h   DPCD 354h   brightness/nits
    0                 2                     2
    0                 3                     3
   ...                ...                    ...
    1                8B                    395
    1                8C                    396
    1                8D                    397
    1                8E                    398
    1                8F                    399
    1                90                    400
*/

// set panel backlight
CBIOS_STATUS EDO156_SetBacklight(PCBIOS_VOID pvcbe, CBIOS_MODULE_INDEX DPModuleIndex, CBIOS_U32 BacklightValue)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_STATUS    Status = CBIOS_OK;

    Status = EDO156_SetBacklightUnLock(pcbe, DPModuleIndex, BacklightValue);

    return Status;
}

// eDP panel power & diaplay onoff
CBIOS_STATUS EDO156_OnOff(PCBIOS_VOID pvcbe, CBIOS_MODULE_INDEX DPModuleIndex, CBIOS_BOOL bTurnOn, CBIOS_U32 BacklightValue)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_STATUS Status = CBIOS_ER_NOT_YET_IMPLEMENTED;

    if(bTurnOn)
    {
        Status = EDO156_SetBacklightUnLock(pvcbe, DPModuleIndex, BacklightValue);   //set max backlight when turn on
    }
    else
    {
        Status = EDO156_SetBacklightUnLock(pvcbe, DPModuleIndex, 0);   //set min backlight when turn off
    }

    return Status;
}

CBIOS_EDP_PANEL_DESC EDO156_Panel_Desc =
{
    /*.VersionNum = */CBIOS_EDP_VERSION,
    /*.MonitorID = */"MST4119",
    /*.EDPPreCaps =*/
    {
        /*.IsNeedPreInit = */CBIOS_FALSE,
        /*.pFnEDPPanelPreInit =*/ EDO156_PreInit,
    },
    /*.EDPCaps = */
    {
        /*.LinkSpeed = */5400000,
        /*.LaneNum = */4,
        /*.BacklightMax = */255,
        /*.BacklightMin = */0,
        /*.Flags = */0x1,//backlight control = 0, use hard code link para
    },
    /*.pFnEDPPanelInit = */EDO156_Init,
    /*.pFnEDPPanelDeInit = */EDO156_DeInit,
    /*.pFnEDPPanelOnOff = */EDO156_OnOff,
    /*.pFnEDPPanelSetBacklight = */EDO156_SetBacklight,
    /*.pFnEDPPanelGetBacklight = */EDO156_GetBacklight,
};

CBIOS_EDP_PANEL_DESC DHUDH_Panel_Desc =
{
    /*.VersionNum = */CBIOS_EDP_VERSION,
    /*.MonitorID = */"MTK1122",
    /*.EDPPreCaps =*/
    {
        /*.IsNeedPreInit = */CBIOS_FALSE,
        /*.pFnEDPPanelPreInit =*/ EDO156_PreInit,
    },
    /*.EDPCaps = */
    {
        /*.LinkSpeed = */5400000,
        /*.LaneNum = */4,
        /*.BacklightMax = */255,
        /*.BacklightMin = */0,
        /*.Flags = */0x1,//backlight control = 0, use hard code link para
    },
    /*.pFnEDPPanelInit = */EDO156_Init,
    /*.pFnEDPPanelDeInit = */EDO156_DeInit,
    /*.pFnEDPPanelOnOff = */EDO156_OnOff,
    /*.pFnEDPPanelSetBacklight = */EDO156_SetBacklight,
    /*.pFnEDPPanelGetBacklight = */EDO156_GetBacklight,
};
