/*
* Copyright 2019-2020 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

/*****************************************************************************
** DESCRIPTION:
** CBios hw initialization functions, mainly POST and chip initialize.
**
** NOTE:
** The functions in this file are hw layer internal functions, 
** CAN ONLY be called by files under Hw folder. 
******************************************************************************/

#include "CBiosChipShare.h"
#include "../CBiosHwShare.h"
#include "../HwBlock/CBiosDIU_HDAC.h"

#define FPGA_DDR2          CBIOS_FALSE
#define CHX001_QT_DDR4_2CH        CBIOS_FALSE
#define CHX004_QT_2D4N            CBIOS_TRUE
#define E3K_QT_8G_3CH_A1          CBIOS_FALSE

#define CBIOS_CLOCK_MAX   1200
#define CBIOS_CLOCK_MIN   200
#define CBIOS_ECLK_DEFAULT 450
#define CBIOS_VCLK_DEFAULT 450

#define E3K_PMPTAG_OFFSET    0x70020
#define E3K_PMPVER_OFFSET    0x70024
#define E3K_PMPTAG_SIZE    4
#define E3K_PMPVER_SIZE    64




#if  CHX004_QT_2D4N
static MMIOREGISTER Mclk_300_Timing_Desktop[] = {
    { 0x0850c,  0x00000002, 32 },
    { 0x053a0,  0x01001600, 32 },
    { 0x053a8,  0x01001600, 32 },
    { 0x05c40,  0x71310000, 32 },
    { 0x05c44,  0x00317111, 32 },
    { 0x05b04,  0x00007344, 32 },
    { 0x05b14,  0x73440000, 32 },
    { 0x0539c,  0x35000000, 32 },
    { 0x053a4,  0x35040008, 32 },
    { 0x053ec,  0x0000000a, 32 },
    { 0x053ec,  0x0000000f, 32 },
    { 0x0539c,  0x35000000, 32 },
    { 0x053a4,  0x35040008, 32 },
    { 0x05b04,  0x00007344, 32 },
    { 0x05b14,  0x73440000, 32 },
    { 0x05c40,  0x71310000, 32 },
    { 0x05c44,  0x00317111, 32 },
    { 0x053ec,  0x000000ff, 32 },
    { 0x05000,  0x31b20000, 32 },
    { 0x0500c,  0x00000000, 32 },
    { 0x05078,  0x00000b00, 32 },
    { 0x050fc,  0x00000008, 32 },
    { 0x05100,  0x0b0d1113, 32 },
    { 0x05104,  0x02030507, 32 },
    { 0x05108,  0x08060403, 32 },
    { 0x0510c,  0x14120e0c, 32 },
    { 0x05110,  0x0c0e1214, 32 },
    { 0x05114,  0x03040608, 32 },
    { 0x05118,  0x07050302, 32 },
    { 0x0511c,  0x13110d0b, 32 },
    { 0x05120,  0x00000000, 32 },
    { 0x05124,  0x00000000, 32 },
    { 0x05128,  0x00000000, 32 },
    { 0x0512c,  0x00000000, 32 },
    { 0x05130,  0x00000000, 32 },
    { 0x05134,  0x00000000, 32 },
    { 0x05138,  0x00000000, 32 },
    { 0x0513c,  0x00000000, 32 },
    { 0x05148,  0xfedcba98, 32 },
    { 0x0514c,  0x00b00000, 32 },
    { 0x05150,  0xffc00011, 32 },
    { 0x05154,  0x00000191, 32 },
    { 0x05158,  0x07140020, 32 },
    { 0x0515c,  0x049d0000, 32 },
    { 0x05160,  0x00000044, 32 },
    { 0x05164,  0x1c0a0100, 32 },
    { 0x05168,  0x88364200, 32 },
    { 0x0516c,  0x08000021, 32 },
    { 0x05170,  0xff7f0100, 32 },
    { 0x05174,  0x00006062, 32 },
    { 0x05178,  0xa000ffd0, 32 },
    { 0x0517c,  0x00000000, 32 },
    { 0x05180,  0x674a4d67, 32 },
    { 0x05184,  0x4382d44c, 32 },
    { 0x05188,  0xa001a650, 32 },
    { 0x0518c,  0x00008014, 32 },
    { 0x05190,  0x01c44e4e, 32 },
    { 0x05194,  0x00104130, 32 },
    { 0x05198,  0x00040000, 32 },
    { 0x0519c,  0x20090000, 32 },
    { 0x051a0,  0x000f3000, 32 },
    { 0x051a4,  0x00000000, 32 },
    { 0x051a8,  0x00440000, 32 },
    { 0x051ac,  0x00000000, 32 },
    { 0x051b0,  0x00000000, 32 },
    { 0x051b4,  0xe0000000, 32 },
    { 0x051b8,  0x00000000, 32 },
    { 0x051bc,  0x00000000, 32 },
    { 0x051c0,  0x00000000, 32 },
    { 0x051c4,  0x00000000, 32 },
    { 0x051c8,  0x02030000, 32 },
    { 0x051cc,  0x00001000, 32 },
    { 0x051d8,  0x00000000, 32 },
    { 0x051dc,  0x00000000, 32 },
    { 0x051e0,  0xf0000000, 32 },
    { 0x051f0,  0x00000000, 32 },
    { 0x051f4,  0x000000ff, 32 },
    { 0x051f8,  0x00000060, 32 },
    { 0x05218,  0x00000000, 32 },
    { 0x05220,  0x00000000, 32 },
    { 0x0522c,  0x00000000, 32 },
    { 0x05230,  0x00000000, 32 },
    { 0x05234,  0x000000ff, 32 },
    { 0x05238,  0x20000000, 32 },
    { 0x0523c,  0x00000000, 32 },
    { 0x05240,  0x00000001, 32 },
    { 0x05244,  0x00100100, 32 },
    { 0x05248,  0xff01ff01, 32 },
    { 0x0524c,  0x00000000, 32 },
    { 0x05250,  0x00000000, 32 },
    { 0x05254,  0x00000000, 32 },
    { 0x05258,  0x00000000, 32 },
    { 0x0525c,  0x00000000, 32 },
    { 0x05260,  0x00000000, 32 },
    { 0x05264,  0x00000000, 32 },
    { 0x05268,  0x00000000, 32 },
    { 0x0526c,  0x00000000, 32 },
    { 0x05270,  0x00000000, 32 },
    { 0x05274,  0x00000000, 32 },
    { 0x05278,  0x00000000, 32 },
    { 0x0527c,  0x00000000, 32 },
    { 0x05280,  0x00000000, 32 },
    { 0x05284,  0x00000000, 32 },
    { 0x05288,  0x00000000, 32 },
    { 0x0528c,  0x00000000, 32 },
    { 0x05290,  0x00000000, 32 },
    { 0x05294,  0x00000000, 32 },
    { 0x05298,  0x00000000, 32 },
    { 0x0529c,  0x00000000, 32 },
    { 0x052ac,  0x00000000, 32 },
    { 0x052b0,  0x00000000, 32 },
    { 0x052b4,  0x00000000, 32 },
    { 0x052b8,  0x00000000, 32 },
    { 0x052bc,  0x00000000, 32 },
    { 0x052c0,  0x00000000, 32 },
    { 0x052c4,  0x00000000, 32 },
    { 0x052c8,  0x00000000, 32 },
    { 0x052cc,  0x00000000, 32 },
    { 0x052d0,  0x00000000, 32 },
    { 0x052d4,  0x00000000, 32 },
    { 0x052d8,  0x00000000, 32 },
    { 0x052dc,  0x00000000, 32 },
    { 0x052e0,  0x00000000, 32 },
    { 0x052e4,  0x00000000, 32 },
    { 0x052e8,  0x00000000, 32 },
    { 0x052ec,  0x00000000, 32 },
    { 0x052f0,  0x00000000, 32 },
    { 0x052f4,  0x00000000, 32 },
    { 0x052f8,  0xff000000, 32 },
    { 0x052fc,  0x0000ff00, 32 },
    { 0x05300,  0x0000ffff, 32 },
    { 0x05304,  0x0000ffff, 32 },
    { 0x05308,  0xffff0000, 32 },
    { 0x0530c,  0x0000ffff, 32 },
    { 0x05310,  0x00000000, 32 },
    { 0x05314,  0x00000000, 32 },
    { 0x05318,  0xff000000, 32 },
    { 0x0531c,  0x0000ff00, 32 },
    { 0x05320,  0x0000ffff, 32 },
    { 0x05324,  0x0000ffff, 32 },
    { 0x05328,  0xffff0000, 32 },
    { 0x0532c,  0x0000ffff, 32 },
    { 0x05330,  0x00000000, 32 },
    { 0x05334,  0x00000000, 32 },
    { 0x05338,  0x00000000, 32 },
    { 0x05380,  0xfc000000, 32 },
    { 0x05384,  0x18180000, 32 },
    { 0x0538c,  0x00000000, 32 },
    { 0x05394,  0x00080000, 32 },
    { 0x05398,  0x00000001, 32 },
    { 0x0539c,  0x35000000, 32 },
    { 0x053a0,  0x01001600, 32 },
    { 0x053a4,  0x35040008, 32 },
    { 0x053a8,  0x01001600, 32 },
    { 0x053ac,  0x00100800, 32 },
    { 0x053b0,  0x00000000, 32 },
    { 0x053b4,  0x00000000, 32 },
    { 0x053ec,  0x000000ff, 32 },
    { 0x05400,  0xd82898a8, 32 },
    { 0x05404,  0x1828d828, 32 },
    { 0x05408,  0x18281828, 32 },
    { 0x0540c,  0x18281828, 32 },
    { 0x05410,  0x98a81828, 32 },
    { 0x05414,  0xd828d828, 32 },
    { 0x05418,  0x18281828, 32 },
    { 0x0541c,  0x18281828, 32 },
    { 0x05420,  0x18281828, 32 },
    { 0x05424,  0x2828a828, 32 },
    { 0x05428,  0x28282828, 32 },
    { 0x0542c,  0xa8282800, 32 },
    { 0x05430,  0x28282828, 32 },
    { 0x05434,  0x28002828, 32 },
    { 0x05438,  0x18189818, 32 },
    { 0x0543c,  0x18181818, 32 },
    { 0x05440,  0x98181800, 32 },
    { 0x05444,  0x18181818, 32 },
    { 0x05448,  0x18001818, 32 },
    { 0x0546c,  0x3838b838, 32 },
    { 0x05470,  0x38383838, 32 },
    { 0x05474,  0x38383800, 32 },
    { 0x05478,  0x38383838, 32 },
    { 0x0547c,  0x38003838, 32 },
    { 0x05480,  0x38383838, 32 },
    { 0x05484,  0x38383838, 32 },
    { 0x05488,  0x38383800, 32 },
    { 0x0548c,  0x38383838, 32 },
    { 0x05490,  0x38003838, 32 },
    { 0x05494,  0x3838b838, 32 },
    { 0x05498,  0x38383838, 32 },
    { 0x0549c,  0x38383800, 32 },
    { 0x054a0,  0x38383838, 32 },
    { 0x054a4,  0x38003838, 32 },
    { 0x054a8,  0x38383838, 32 },
    { 0x054ac,  0x38383838, 32 },
    { 0x054b0,  0x38383800, 32 },
    { 0x054b4,  0x38383838, 32 },
    { 0x054b8,  0x38003838, 32 },
    { 0x054fc,  0x1010d010, 32 },
    { 0x05500,  0x10101010, 32 },
    { 0x05504,  0x10101010, 32 },
    { 0x05508,  0x10101010, 32 },
    { 0x0550c,  0x10101010, 32 },
    { 0x05510,  0x10101010, 32 },
    { 0x05514,  0x10101010, 32 },
    { 0x05518,  0x10101010, 32 },
    { 0x0551c,  0x10101010, 32 },
    { 0x05520,  0x10101010, 32 },
    { 0x05524,  0x10101010, 32 },
    { 0x05528,  0x10101010, 32 },
    { 0x0552c,  0x10101010, 32 },
    { 0x05530,  0x10101010, 32 },
    { 0x05534,  0x10101010, 32 },
    { 0x05538,  0x10101010, 32 },
    { 0x0553c,  0x10101010, 32 },
    { 0x05540,  0x10101010, 32 },
    { 0x05544,  0x10101010, 32 },
    { 0x05548,  0x10101010, 32 },
    { 0x0554c,  0x10101010, 32 },
    { 0x05550,  0x10101010, 32 },
    { 0x05554,  0x10101010, 32 },
    { 0x05558,  0x10101010, 32 },
    { 0x0555c,  0x10101010, 32 },
    { 0x05560,  0x10101010, 32 },
    { 0x05564,  0x10101010, 32 },
    { 0x05568,  0x10101010, 32 },
    { 0x0556c,  0x10101010, 32 },
    { 0x05570,  0x10101010, 32 },
    { 0x05574,  0x10101010, 32 },
    { 0x05578,  0x10101010, 32 },
    { 0x0557c,  0x10101010, 32 },
    { 0x05580,  0x10101010, 32 },
    { 0x05584,  0x10101010, 32 },
    { 0x05588,  0x10101010, 32 },
    { 0x0558c,  0x10101010, 32 },
    { 0x05590,  0x10101010, 32 },
    { 0x05594,  0x10101010, 32 },
    { 0x05598,  0x10101010, 32 },
    { 0x0559c,  0x10101010, 32 },
    { 0x055a0,  0x10101010, 32 },
    { 0x055a4,  0x10101010, 32 },
    { 0x055a8,  0x10101010, 32 },
    { 0x055ac,  0x10101010, 32 },
    { 0x055b0,  0x10101010, 32 },
    { 0x055b4,  0x10101010, 32 },
    { 0x055b8,  0x10101010, 32 },
    { 0x055bc,  0x10101010, 32 },
    { 0x055c0,  0x10101010, 32 },
    { 0x055c4,  0x10101010, 32 },
    { 0x055c8,  0x10101010, 32 },
    { 0x055cc,  0x10101010, 32 },
    { 0x055d0,  0x10101010, 32 },
    { 0x055d4,  0x10101010, 32 },
    { 0x055d8,  0x10101010, 32 },
    { 0x055dc,  0x10101010, 32 },
    { 0x055e0,  0x10101010, 32 },
    { 0x055e4,  0x10101010, 32 },
    { 0x055e8,  0x10101010, 32 },
    { 0x055ec,  0x10101010, 32 },
    { 0x055f0,  0x10101010, 32 },
    { 0x055f4,  0x10101010, 32 },
    { 0x055f8,  0x10101010, 32 },
    { 0x055fc,  0x10101010, 32 },
    { 0x05600,  0x10101010, 32 },
    { 0x05604,  0x10101010, 32 },
    { 0x05608,  0x10101010, 32 },
    { 0x0560c,  0x10101010, 32 },
    { 0x05610,  0x10101010, 32 },
    { 0x05614,  0x10101010, 32 },
    { 0x05618,  0x10101010, 32 },
    { 0x0561c,  0x10101010, 32 },
    { 0x05620,  0x10101010, 32 },
    { 0x05624,  0x10101010, 32 },
    { 0x05628,  0x10101010, 32 },
    { 0x0562c,  0x10101010, 32 },
    { 0x05630,  0x10101010, 32 },
    { 0x05634,  0x10101010, 32 },
    { 0x05638,  0x10101010, 32 },
    { 0x0563c,  0x10101010, 32 },
    { 0x05640,  0x10101010, 32 },
    { 0x05644,  0x10101010, 32 },
    { 0x05648,  0x10101010, 32 },
    { 0x0564c,  0x10101010, 32 },
    { 0x05650,  0x10101010, 32 },
    { 0x05654,  0x10101010, 32 },
    { 0x05658,  0x10101010, 32 },
    { 0x0565c,  0x10101010, 32 },
    { 0x05660,  0x10101010, 32 },
    { 0x05664,  0x10101010, 32 },
    { 0x05668,  0x10101010, 32 },
    { 0x0566c,  0x10101010, 32 },
    { 0x05670,  0x10101010, 32 },
    { 0x05674,  0x10101010, 32 },
    { 0x05678,  0x10101010, 32 },
    { 0x0567c,  0x10101010, 32 },
    { 0x05680,  0x10101010, 32 },
    { 0x05684,  0x10101010, 32 },
    { 0x05688,  0x10101010, 32 },
    { 0x0568c,  0x10101010, 32 },
    { 0x05690,  0x10101010, 32 },
    { 0x05694,  0x10101010, 32 },
    { 0x05698,  0x10101010, 32 },
    { 0x0569c,  0x10101010, 32 },
    { 0x056a0,  0x10101010, 32 },
    { 0x056a4,  0x10101010, 32 },
    { 0x056a8,  0x10101010, 32 },
    { 0x056ac,  0x10101010, 32 },
    { 0x056b0,  0x10101010, 32 },
    { 0x056b4,  0x10101010, 32 },
    { 0x056b8,  0x10101010, 32 },
    { 0x056bc,  0x10101010, 32 },
    { 0x056c0,  0x10101010, 32 },
    { 0x056c4,  0x10101010, 32 },
    { 0x056c8,  0x10101010, 32 },
    { 0x056cc,  0x10101010, 32 },
    { 0x056d0,  0x10101010, 32 },
    { 0x056d4,  0x10101010, 32 },
    { 0x056d8,  0x10101010, 32 },
    { 0x056dc,  0x10101010, 32 },
    { 0x056e0,  0x10101010, 32 },
    { 0x056e4,  0x10101010, 32 },
    { 0x056e8,  0x10101010, 32 },
    { 0x056ec,  0x10101010, 32 },
    { 0x056f0,  0x10101010, 32 },
    { 0x056f4,  0x10101010, 32 },
    { 0x056f8,  0x10101010, 32 },
    { 0x056fc,  0x10101010, 32 },
    { 0x05700,  0x10101010, 32 },
    { 0x05704,  0x10101010, 32 },
    { 0x05708,  0x10101010, 32 },
    { 0x0570c,  0x10101010, 32 },
    { 0x05710,  0x10101010, 32 },
    { 0x05714,  0x10101010, 32 },
    { 0x05718,  0x10101010, 32 },
    { 0x0571c,  0x10101010, 32 },
    { 0x05720,  0x10101010, 32 },
    { 0x05724,  0x10101010, 32 },
    { 0x05728,  0x10101010, 32 },
    { 0x0572c,  0x10101010, 32 },
    { 0x05730,  0x10101010, 32 },
    { 0x05734,  0x10101010, 32 },
    { 0x05738,  0x10101010, 32 },
    { 0x0573c,  0x1010d010, 32 },
    { 0x05740,  0x10101010, 32 },
    { 0x05744,  0x10101010, 32 },
    { 0x05748,  0x10101010, 32 },
    { 0x0574c,  0x10101010, 32 },
    { 0x05750,  0x10101010, 32 },
    { 0x05754,  0x10101010, 32 },
    { 0x05758,  0x10101010, 32 },
    { 0x0575c,  0x10101010, 32 },
    { 0x05760,  0x10101010, 32 },
    { 0x05764,  0x10101010, 32 },
    { 0x05768,  0x10101010, 32 },
    { 0x0576c,  0x10101010, 32 },
    { 0x05770,  0x10101010, 32 },
    { 0x05774,  0x10101010, 32 },
    { 0x05778,  0x10101010, 32 },
    { 0x0577c,  0x10101010, 32 },
    { 0x05780,  0x10101010, 32 },
    { 0x05784,  0x10101010, 32 },
    { 0x05788,  0x10101010, 32 },
    { 0x0578c,  0x10101010, 32 },
    { 0x05790,  0x10101010, 32 },
    { 0x05794,  0x10101010, 32 },
    { 0x05798,  0x10101010, 32 },
    { 0x0579c,  0x10101010, 32 },
    { 0x057a0,  0x10101010, 32 },
    { 0x057a4,  0x10101010, 32 },
    { 0x057a8,  0x10101010, 32 },
    { 0x057ac,  0x10101010, 32 },
    { 0x057b0,  0x10101010, 32 },
    { 0x057b4,  0x10101010, 32 },
    { 0x057b8,  0x10101010, 32 },
    { 0x057bc,  0x10101010, 32 },
    { 0x057c0,  0x10101010, 32 },
    { 0x057c4,  0x10101010, 32 },
    { 0x057c8,  0x10101010, 32 },
    { 0x057cc,  0x10101010, 32 },
    { 0x057d0,  0x10101010, 32 },
    { 0x057d4,  0x10101010, 32 },
    { 0x057d8,  0x10101010, 32 },
    { 0x057dc,  0x10101010, 32 },
    { 0x057e0,  0x10101010, 32 },
    { 0x057e4,  0x10101010, 32 },
    { 0x057e8,  0x10101010, 32 },
    { 0x057ec,  0x10101010, 32 },
    { 0x057f0,  0x10101010, 32 },
    { 0x057f4,  0x10101010, 32 },
    { 0x057f8,  0x10101010, 32 },
    { 0x057fc,  0x10101010, 32 },
    { 0x05800,  0x10101010, 32 },
    { 0x05804,  0x10101010, 32 },
    { 0x05808,  0x10101010, 32 },
    { 0x0580c,  0x10101010, 32 },
    { 0x05810,  0x10101010, 32 },
    { 0x05814,  0x10101010, 32 },
    { 0x05818,  0x10101010, 32 },
    { 0x0581c,  0x10101010, 32 },
    { 0x05820,  0x10101010, 32 },
    { 0x05824,  0x10101010, 32 },
    { 0x05828,  0x10101010, 32 },
    { 0x0582c,  0x10101010, 32 },
    { 0x05830,  0x10101010, 32 },
    { 0x05834,  0x10101010, 32 },
    { 0x05838,  0x10101010, 32 },
    { 0x0583c,  0x10101010, 32 },
    { 0x05840,  0x10101010, 32 },
    { 0x05844,  0x10101010, 32 },
    { 0x05848,  0x10101010, 32 },
    { 0x0584c,  0x10101010, 32 },
    { 0x05850,  0x10101010, 32 },
    { 0x05854,  0x10101010, 32 },
    { 0x05858,  0x10101010, 32 },
    { 0x0585c,  0x10101010, 32 },
    { 0x05860,  0x10101010, 32 },
    { 0x05864,  0x10101010, 32 },
    { 0x05868,  0x10101010, 32 },
    { 0x0586c,  0x10101010, 32 },
    { 0x05870,  0x10101010, 32 },
    { 0x05874,  0x10101010, 32 },
    { 0x05878,  0x10101010, 32 },
    { 0x0587c,  0x10101010, 32 },
    { 0x05880,  0x10101010, 32 },
    { 0x05884,  0x10101010, 32 },
    { 0x05888,  0x10101010, 32 },
    { 0x0588c,  0x10101010, 32 },
    { 0x05890,  0x10101010, 32 },
    { 0x05894,  0x10101010, 32 },
    { 0x05898,  0x10101010, 32 },
    { 0x0589c,  0x10101010, 32 },
    { 0x058a0,  0x10101010, 32 },
    { 0x058a4,  0x10101010, 32 },
    { 0x058a8,  0x10101010, 32 },
    { 0x058ac,  0x10101010, 32 },
    { 0x058b0,  0x10101010, 32 },
    { 0x058b4,  0x10101010, 32 },
    { 0x058b8,  0x10101010, 32 },
    { 0x058bc,  0x10101010, 32 },
    { 0x058c0,  0x10101010, 32 },
    { 0x058c4,  0x10101010, 32 },
    { 0x058c8,  0x10101010, 32 },
    { 0x058cc,  0x10101010, 32 },
    { 0x058d0,  0x10101010, 32 },
    { 0x058d4,  0x10101010, 32 },
    { 0x058d8,  0x10101010, 32 },
    { 0x058dc,  0x10101010, 32 },
    { 0x058e0,  0x10101010, 32 },
    { 0x058e4,  0x10101010, 32 },
    { 0x058e8,  0x10101010, 32 },
    { 0x058ec,  0x10101010, 32 },
    { 0x058f0,  0x10101010, 32 },
    { 0x058f4,  0x10101010, 32 },
    { 0x058f8,  0x10101010, 32 },
    { 0x058fc,  0x10101010, 32 },
    { 0x05900,  0x10101010, 32 },
    { 0x05904,  0x10101010, 32 },
    { 0x05908,  0x10101010, 32 },
    { 0x0590c,  0x10101010, 32 },
    { 0x05910,  0x10101010, 32 },
    { 0x05914,  0x10101010, 32 },
    { 0x05918,  0x10101010, 32 },
    { 0x0591c,  0x10101010, 32 },
    { 0x05920,  0x10101010, 32 },
    { 0x05924,  0x10101010, 32 },
    { 0x05928,  0x10101010, 32 },
    { 0x0592c,  0x10101010, 32 },
    { 0x05930,  0x10101010, 32 },
    { 0x05934,  0x10101010, 32 },
    { 0x05938,  0x10101010, 32 },
    { 0x0593c,  0x10101010, 32 },
    { 0x05940,  0x10101010, 32 },
    { 0x05944,  0x10101010, 32 },
    { 0x05948,  0x10101010, 32 },
    { 0x0594c,  0x10101010, 32 },
    { 0x05950,  0x10101010, 32 },
    { 0x05954,  0x10101010, 32 },
    { 0x05958,  0x10101010, 32 },
    { 0x0595c,  0x10101010, 32 },
    { 0x05960,  0x10101010, 32 },
    { 0x05964,  0x10101010, 32 },
    { 0x05968,  0x10101010, 32 },
    { 0x0596c,  0x10101010, 32 },
    { 0x05970,  0x10101010, 32 },
    { 0x05974,  0x10101010, 32 },
    { 0x05978,  0x10101010, 32 },
    { 0x0597c,  0x88888888, 32 },
    { 0x05980,  0x88888888, 32 },
    { 0x05984,  0x88888888, 32 },
    { 0x05988,  0x88888888, 32 },
    { 0x0598c,  0x88888888, 32 },
    { 0x05990,  0x88888888, 32 },
    { 0x05994,  0x88888888, 32 },
    { 0x05998,  0x88888888, 32 },
    { 0x0599c,  0x88888888, 32 },
    { 0x059a0,  0x88888888, 32 },
    { 0x059a4,  0x88888888, 32 },
    { 0x059a8,  0x88888888, 32 },
    { 0x059ac,  0x88888888, 32 },
    { 0x059b0,  0x88888888, 32 },
    { 0x059b4,  0x88888888, 32 },
    { 0x059b8,  0x88888888, 32 },
    { 0x059bc,  0x88888888, 32 },
    { 0x059c0,  0x88888888, 32 },
    { 0x059c4,  0x00000000, 32 },
    { 0x059c8,  0x00000000, 32 },
    { 0x059cc,  0x00000000, 32 },
    { 0x059d0,  0x00000000, 32 },
    { 0x059d4,  0x00000000, 32 },
    { 0x059d8,  0x00000000, 32 },
    { 0x059dc,  0x00000000, 32 },
    { 0x059e0,  0x00000000, 32 },
    { 0x059e4,  0x00000000, 32 },
    { 0x059e8,  0x00000000, 32 },
    { 0x059ec,  0x00000000, 32 },
    { 0x059f0,  0x00000000, 32 },
    { 0x059f4,  0x00000000, 32 },
    { 0x059f8,  0x00000000, 32 },
    { 0x059fc,  0x00000000, 32 },
    { 0x05a00,  0x00000000, 32 },
    { 0x05a04,  0x00000000, 32 },
    { 0x05a08,  0x00000000, 32 },
    { 0x05a0c,  0x33333333, 32 },
    { 0x05a10,  0x33330003, 32 },
    { 0x05a14,  0x00033333, 32 },
    { 0x05a18,  0x33333333, 32 },
    { 0x05a1c,  0x33333333, 32 },
    { 0x05a20,  0x00033333, 32 },
    { 0x05a24,  0x33333333, 32 },
    { 0x05a28,  0x33330003, 32 },
    { 0x05a2c,  0x00033333, 32 },
    { 0x05a30,  0x33333333, 32 },
    { 0x05a34,  0x33330003, 32 },
    { 0x05a38,  0x00033333, 32 },
    { 0x05a3c,  0x33333333, 32 },
    { 0x05a40,  0x33333333, 32 },
    { 0x05a44,  0x33330033, 32 },
    { 0x05a48,  0x33333333, 32 },
    { 0x05a4c,  0x00333333, 32 },
    { 0x05a50,  0x00000000, 32 },
    { 0x05a54,  0x00000000, 32 },
    { 0x05a58,  0x00000000, 32 },
    { 0x05a5c,  0x00000000, 32 },
    { 0x05a60,  0x00000000, 32 },
    { 0x05a64,  0x00000000, 32 },
    { 0x05a68,  0x00000000, 32 },
    { 0x05a6c,  0x00000000, 32 },
    { 0x05a70,  0x00000000, 32 },
    { 0x05a74,  0x00000000, 32 },
    { 0x05a78,  0x00000000, 32 },
    { 0x05a7c,  0x00000000, 32 },
    { 0x05a80,  0x00000000, 32 },
    { 0x05a84,  0x00000000, 32 },
    { 0x05a88,  0x00000000, 32 },
    { 0x05a8c,  0x00000000, 32 },
    { 0x05a90,  0x00000000, 32 },
    { 0x05a94,  0x00000000, 32 },
    { 0x05a98,  0x00000000, 32 },
    { 0x05a9c,  0x00000000, 32 },
    { 0x05aa0,  0x00000000, 32 },
    { 0x05aa4,  0x00000000, 32 },
    { 0x05aa8,  0x00000000, 32 },
    { 0x05aac,  0x00000000, 32 },
    { 0x05ab0,  0x00000000, 32 },
    { 0x05ab4,  0x00000000, 32 },
    { 0x05ab8,  0x00000000, 32 },
    { 0x05abc,  0x00000000, 32 },
    { 0x05ac0,  0x00000000, 32 },
    { 0x05ac4,  0x00000000, 32 },
    { 0x05ac8,  0x00000000, 32 },
    { 0x05acc,  0x00000000, 32 },
    { 0x05ad0,  0x00000000, 32 },
    { 0x05ad4,  0x00000000, 32 },
    { 0x05ad8,  0x00000000, 32 },
    { 0x05adc,  0x00000000, 32 },
    { 0x05ae0,  0x00000000, 32 },
    { 0x05ae4,  0x00000000, 32 },
    { 0x05ae8,  0x00000000, 32 },
    { 0x05aec,  0x00000000, 32 },
    { 0x05af0,  0x00000000, 32 },
    { 0x05af4,  0x00000000, 32 },
    { 0x05af8,  0x00000000, 32 },
    { 0x05afc,  0x00000000, 32 },
    { 0x05b00,  0x00000000, 32 },
    { 0x05b04,  0x00007344, 32 },
    { 0x05b14,  0x73440000, 32 },
    { 0x05b24,  0x00000000, 32 },
    { 0x05b28,  0x00000000, 32 },
    { 0x05b2c,  0x00000000, 32 },
    { 0x05b30,  0x00000000, 32 },
    { 0x05b34,  0x00000000, 32 },
    { 0x05b3c,  0x39003900, 32 },
    { 0x05b40,  0x39003900, 32 },
    { 0x05b44,  0x39003900, 32 },
    { 0x05b48,  0x39003900, 32 },
    { 0x05b4c,  0x39003900, 32 },
    { 0x05b50,  0x39003900, 32 },
    { 0x05b54,  0x39003900, 32 },
    { 0x05b58,  0x39003900, 32 },
    { 0x05b5c,  0x39003900, 32 },
    { 0x05b60,  0x0db64db6, 32 },
    { 0x05b64,  0x0db60db6, 32 },
    { 0x05b68,  0x0db60db6, 32 },
    { 0x05b6c,  0x0db60db6, 32 },
    { 0x05b70,  0x4db60db6, 32 },
    { 0x05b74,  0x0db60db6, 32 },
    { 0x05b78,  0x0db60db6, 32 },
    { 0x05b7c,  0x0db60db6, 32 },
    { 0x05b80,  0x0db60db6, 32 },
    { 0x05b84,  0x00000800, 32 },
    { 0x05b88,  0x00000000, 32 },
    { 0x05b8c,  0x08000000, 32 },
    { 0x05b90,  0x00000000, 32 },
    { 0x05b94,  0x00000000, 32 },
    { 0x05bb0,  0x00000c4c, 32 },
    { 0x05bb4,  0x00415050, 32 },
    { 0x05bb8,  0x00000c4c, 32 },
    { 0x05bbc,  0x00415050, 32 },
    { 0x05bc0,  0x00000000, 32 },
    { 0x05bc4,  0x00000000, 32 },
    { 0x05bc8,  0x34000000, 32 },
    { 0x05bd0,  0x01000000, 32 },
    { 0x05bd4,  0x00004c30, 32 },
    { 0x05bd8,  0x07d00020, 32 },
    { 0x05bdc,  0xff040064, 32 },
    { 0x05be0,  0x000feb0f, 32 },
    { 0x05be4,  0x000093b6, 32 },
    { 0x05be8,  0x000ed063, 32 },
    { 0x05bec,  0x00018d92, 32 },
    { 0x05bf0,  0x000f1b9f, 32 },
    { 0x05c08,  0x00000000, 32 },
    { 0x05c0c,  0x00000000, 32 },
    { 0x05c10,  0x00000000, 32 },
    { 0x05c14,  0x00000000, 32 },
    { 0x05c18,  0x00000000, 32 },
    { 0x05c1c,  0x00000000, 32 },
    { 0x05c20,  0x00000000, 32 },
    { 0x05c24,  0x00000000, 32 },
    { 0x05c2c,  0x00000003, 32 },
    { 0x05c30,  0x88880000, 32 },
    { 0x05c34,  0x00000000, 32 },
    { 0x05c38,  0x88880000, 32 },
    { 0x05c3c,  0x00000000, 32 },
    { 0x05c40,  0x71310000, 32 },
    { 0x05c44,  0x00317111, 32 },
    { 0x05c48,  0x03eb0000, 32 },
    { 0x05c4c,  0x1423320f, 32 },
    { 0x05c50,  0x000319c8, 32 },
    { 0x05c54,  0x00030003, 32 },
    { 0x05c58,  0x00030003, 32 },
    { 0x05c5c,  0x00030003, 32 },
    { 0x05c60,  0x00030003, 32 },
    { 0x05c64,  0x00030003, 32 },
    { 0x05c68,  0x00330003, 32 },
    { 0x05c6c,  0x00200000, 32 },
    { 0x05c70,  0x00000000, 32 },
    { 0x05c74,  0x00000000, 32 },
    { 0x05c78,  0x00000000, 32 },
    { 0x05c7c,  0x00000000, 32 },
    { 0x05c80,  0x00000000, 32 },
    { 0x05c84,  0x00000000, 32 },
    { 0x05c88,  0x00002000, 32 },
    { 0x05c8c,  0x00000000, 32 },
    { 0x05c90,  0x00000000, 32 },
    { 0x05c94,  0x00000000, 32 },
    { 0x05c98,  0x00000000, 32 },
    { 0x05c9c,  0x00000000, 32 },
    { 0x05ca0,  0x00000000, 32 },
    { 0x05ca4,  0x00000000, 32 },
    { 0x05ca8,  0x00000000, 32 },
    { 0x05cac,  0x00000000, 32 },
    { 0x05cb0,  0x00000000, 32 },
    { 0x05cb4,  0x00000000, 32 },
    { 0x05cb8,  0x00000000, 32 },
    { 0x05cbc,  0x00000000, 32 },
    { 0x05cc0,  0x00000000, 32 },
    { 0x05cc4,  0x00030000, 32 },
    { 0x05cc8,  0x00000000, 32 },
    { 0x05194,  0x00104010, 32 },
    { 0x05194,  0x00104130, 32 },
    { 0x05194,  0x40114130, 32 },
    { 0x0514c,  0x00b04000, 32 },
    { 0x05164,  0x140a0000, 32 },
    { 0x05130,  0x08000800, 32 },
    { 0x05134,  0x08000800, 32 },
    { 0x05138,  0x08000800, 32 },
    { 0x0513c,  0x08000800, 32 },
    { 0x05c2c,  0x00000003, 32 },
    { 0x05014,  0x80,8 },
    { 0x05008,  0x00,8 },
    { 0x08af1,  0x01,8 },
    { 0x0850c,  0x02,8 },
    { 0x08aa0,  0x10,8 },
};
#elif E3K_QT_8G_3CH_A1
static MMIOREGISTER Mclk_300_Timing_Desktop[] = {
    { 0x0850c, 0x00000002, 32 },
    { 0x0d004, 0x02010b40, 32 },
    { 0x0d008, 0x02000220, 32 },
    { 0x0d00c, 0x00041800, 32 },
    { 0x0d00c, 0x00050480, 32 },
    { 0x0d00c, 0x00061058, 32 },
    { 0x0d014, 0x010000ff, 32 },
    { 0x0d018, 0x006136d1, 32 },
    { 0x0d01c, 0x0d0d1208, 32 },
    { 0x0d020, 0xc3200423, 32 },
    { 0x0d024, 0x01516a2d, 32 },
    { 0x0d028, 0x00808181, 32 },
    { 0x0d02c, 0x0508d7c1, 32 },
    { 0x0d030, 0x0000d000, 32 },
    { 0x0d034, 0x00006007, 32 },
    { 0x0d038, 0x00100100, 32 },
    { 0x0d03c, 0x0008c2cc, 32 },
    { 0x0d040, 0xff0acf00, 32 },
    { 0x0d044, 0x000008fd, 32 },
    { 0x0d048, 0x1082007a, 32 },
    { 0x0d04c, 0x1082007a, 32 },
    { 0x0d050, 0x1082007a, 32 },
    { 0x0d054, 0x1082007a, 32 },
    { 0x0d148, 0x1082007a, 32 },
    { 0x0d14c, 0x1082007a, 32 },
    { 0x0d150, 0x1082007a, 32 },
    { 0x0d154, 0x1082007a, 32 },
    { 0x0d0ec, 0x20202020, 32 },
    { 0x0d1ec, 0x20202020, 32 },
    { 0x0d058, 0x00999999, 32 },
    { 0x0d158, 0x00888888, 32 },
    { 0x0d05c, 0x44444444, 32 },
    { 0x0d15c, 0x44444444, 32 },
    { 0x0d060, 0x44444444, 32 },
    { 0x0d160, 0x44444444, 32 },
    { 0x0d064, 0x90909090, 32 },
    { 0x0d164, 0x90909090, 32 },
    { 0x0d068, 0x90909090, 32 },
    { 0x0d168, 0x90909090, 32 },
    { 0x0d06c, 0x00022701, 32 },
    { 0x0d128, 0x66040000, 32 },
    { 0x0d070, 0x1a050030, 32 },
    { 0x0d170, 0x20200020, 32 },
    { 0x0d074, 0x20202020, 32 },
    { 0x0d174, 0x20202020, 32 },
    { 0x0d078, 0x35353535, 32 },
    { 0x0d178, 0x35353535, 32 },
    { 0x0d080, 0x20202020, 32 },
    { 0x0d180, 0x20202020, 32 },
    { 0x0d084, 0x00000000, 32 },
    { 0x0d184, 0x00000000, 32 },
    { 0x0d08c, 0x55555555, 32 },
    { 0x0d090, 0x55555555, 32 },
    { 0x0d118, 0x0F0FFF0F, 32 },
    { 0x0d218, 0x0F0FFF0F, 32 },
    { 0x0d114, 0x003fff9f, 32 },
    { 0x0d138, 0x1f040000, 32 },
    { 0x0d114, 0x003fff9f, 32 },
    { 0x0d328, 0x00000009, 32 },
    { 0x0d32c, 0x80007ed6, 32 },
    { 0x0d33c, 0x0040f0ff, 32 },
    { 0x0d340, 0x0110f010, 32 },
    { 0x0d240, 0x00000080, 32 },
    { 0x0d290, 0x84880000, 32 },
    { 0x0d2c0, 0x000000c0, 32 },
    { 0x0d3f0, 0x03000001, 32 },
    { 0x0d000, 0x00000414, 32 },
    { 0x0d000, 0x00000514, 32 },
    { 0x0e004, 0x02010b40, 32 },
    { 0x0e008, 0x02000220, 32 },
    { 0x0e00c, 0x00041800, 32 },
    { 0x0e00c, 0x00050480, 32 },
    { 0x0e00c, 0x00061058, 32 },
    { 0x0e014, 0x010000ff, 32 },
    { 0x0e018, 0x006136d1, 32 },
    { 0x0e01c, 0x0d0d1208, 32 },
    { 0x0e020, 0xc3200423, 32 },
    { 0x0e024, 0x01516a2d, 32 },
    { 0x0e028, 0x00808181, 32 },
    { 0x0e02c, 0x0508d7c1, 32 },
    { 0x0e030, 0x0000d000, 32 },
    { 0x0e034, 0x00006007, 32 },
    { 0x0e038, 0x00100100, 32 },
    { 0x0e03c, 0x0008c2cc, 32 },
    { 0x0e040, 0xff0acf00, 32 },
    { 0x0e044, 0x000008fd, 32 },
    { 0x0e048, 0x1082007a, 32 },
    { 0x0e04c, 0x1082007a, 32 },
    { 0x0e050, 0x1082007a, 32 },
    { 0x0e054, 0x1082007a, 32 },
    { 0x0e148, 0x1082007a, 32 },
    { 0x0e14c, 0x1082007a, 32 },
    { 0x0e150, 0x1082007a, 32 },
    { 0x0e154, 0x1082007a, 32 },
    { 0x0e0ec, 0x20202020, 32 },
    { 0x0e1ec, 0x20202020, 32 },
    { 0x0e058, 0x00999999, 32 },
    { 0x0e158, 0x00888888, 32 },
    { 0x0e05c, 0x44444444, 32 },
    { 0x0e15c, 0x44444444, 32 },
    { 0x0e060, 0x44444444, 32 },
    { 0x0e160, 0x44444444, 32 },
    { 0x0e064, 0x90909090, 32 },
    { 0x0e164, 0x90909090, 32 },
    { 0x0e068, 0x90909090, 32 },
    { 0x0e168, 0x90909090, 32 },
    { 0x0e06c, 0x00022701, 32 },
    { 0x0e128, 0x66040000, 32 },
    { 0x0e070, 0x1a050030, 32 },
    { 0x0e170, 0x20200020, 32 },
    { 0x0e074, 0x20202020, 32 },
    { 0x0e174, 0x20202020, 32 },
    { 0x0e078, 0x35353535, 32 },
    { 0x0e178, 0x35353535, 32 },
    { 0x0e080, 0x20202020, 32 },
    { 0x0e180, 0x20202020, 32 },
    { 0x0e084, 0x00000000, 32 },
    { 0x0e184, 0x00000000, 32 },
    { 0x0e08c, 0x55555555, 32 },
    { 0x0e090, 0x55555555, 32 },
    { 0x0e118, 0x0F0FFF0F, 32 },
    { 0x0e218, 0x0F0FFF0F, 32 },
    { 0x0e114, 0x003fff9f, 32 },
    { 0x0e138, 0x1f040000, 32 },
    { 0x0e114, 0x003fff9f, 32 },
    { 0x0e328, 0x00000009, 32 },
    { 0x0e32c, 0x80007ed6, 32 },
    { 0x0e33c, 0x0040f0ff, 32 },
    { 0x0e340, 0x0110f010, 32 },
    { 0x0e240, 0x00000080, 32 },
    { 0x0e290, 0x84880000, 32 },
    { 0x0e2c0, 0x000000c0, 32 },
    { 0x0e3f0, 0x03000001, 32 },
    { 0x0e000, 0x00000414, 32 },
    { 0x0e000, 0x00000514, 32 },
    { 0x0f004, 0x02010b40, 32 },
    { 0x0f008, 0x02000220, 32 },
    { 0x0f00c, 0x00041800, 32 },
    { 0x0f00c, 0x00050480, 32 },
    { 0x0f00c, 0x00061058, 32 },
    { 0x0f014, 0x010000ff, 32 },
    { 0x0f018, 0x006136d1, 32 },
    { 0x0f01c, 0x0d0d1208, 32 },
    { 0x0f020, 0xc3200423, 32 },
    { 0x0f024, 0x01516a2d, 32 },
    { 0x0f028, 0x00808181, 32 },
    { 0x0f02c, 0x0508d7c1, 32 },
    { 0x0f030, 0x0000d000, 32 },
    { 0x0f034, 0x00006007, 32 },
    { 0x0f038, 0x00100100, 32 },
    { 0x0f03c, 0x0008c2cc, 32 },
    { 0x0f040, 0xff0acf00, 32 },
    { 0x0f044, 0x000008fd, 32 },
    { 0x0f048, 0x1082007a, 32 },
    { 0x0f04c, 0x1082007a, 32 },
    { 0x0f050, 0x1082007a, 32 },
    { 0x0f054, 0x1082007a, 32 },
    { 0x0f148, 0x1082007a, 32 },
    { 0x0f14c, 0x1082007a, 32 },
    { 0x0f150, 0x1082007a, 32 },
    { 0x0f154, 0x1082007a, 32 },
    { 0x0f0ec, 0x20202020, 32 },
    { 0x0f1ec, 0x20202020, 32 },
    { 0x0f058, 0x00999999, 32 },
    { 0x0f158, 0x00888888, 32 },
    { 0x0f05c, 0x44444444, 32 },
    { 0x0f15c, 0x44444444, 32 },
    { 0x0f060, 0x44444444, 32 },
    { 0x0f160, 0x44444444, 32 },
    { 0x0f064, 0x90909090, 32 },
    { 0x0f164, 0x90909090, 32 },
    { 0x0f068, 0x90909090, 32 },
    { 0x0f168, 0x90909090, 32 },
    { 0x0f06c, 0x00022701, 32 },
    { 0x0f128, 0x66040000, 32 },
    { 0x0f070, 0x1a050030, 32 },
    { 0x0f170, 0x20200020, 32 },
    { 0x0f074, 0x20202020, 32 },
    { 0x0f174, 0x20202020, 32 },
    { 0x0f078, 0x35353535, 32 },
    { 0x0f178, 0x35353535, 32 },
    { 0x0f080, 0x20202020, 32 },
    { 0x0f180, 0x20202020, 32 },
    { 0x0f084, 0x00000000, 32 },
    { 0x0f184, 0x00000000, 32 },
    { 0x0f08c, 0x55555555, 32 },
    { 0x0f090, 0x55555555, 32 },
    { 0x0f118, 0x0F0FFF0F, 32 },
    { 0x0f218, 0x0F0FFF0F, 32 },
    { 0x0f114, 0x003fff9f, 32 },
    { 0x0f138, 0x1f040000, 32 },
    { 0x0f114, 0x003fff9f, 32 },
    { 0x0f328, 0x00000009, 32 },
    { 0x0f32c, 0x80007ed6, 32 },
    { 0x0f33c, 0x0040f0ff, 32 },
    { 0x0f340, 0x0110f010, 32 },
    { 0x0f240, 0x00000080, 32 },
    { 0x0f290, 0x84880000, 32 },
    { 0x0f2c0, 0x000000c0, 32 },
    { 0x0f3f0, 0x03000001, 32 },
    { 0x0f000, 0x00000414, 32 },
    { 0x0f000, 0x00000514, 32 },
    { 0x0850c, 0x00000002, 32 },
};
#elif QT_DDR4_2CH
static MMIOREGISTER Mclk_300_Timing_Desktop[] = {
    { 0x0000850c, 0x00000002, 32 },
    { 0x00005040, 0x00000000, 32 },
    { 0x00005044, 0x00000000, 32 },
    { 0x00005100, 0xffff1008, 32 },
    { 0x00005104, 0xffff1008, 32 },
    { 0x00005108, 0xffff0800, 32 },
    { 0x0000510c, 0xffff0800, 32 },

    { 0x00005148, 0xc2ff3800, 32 },

    { 0x00005158, 0x27380200, 32 },
    { 0x0000515c, 0x32818069, 32 },
    { 0x00005160, 0x88004040, 32 },
    { 0x00005164, 0x00006024, 32 },
    { 0x00005168, 0x5d184e4e, 32 },
    { 0x0000516c, 0x08104010, 32 },

    { 0x00005250, 0x00ff0000, 32 },
    { 0x0000516c, 0x08104030, 32 },
    { 0x0000516c, 0x48114030, 32 },
    { 0x00005118, 0xba98ba98, 32 },

    { 0x00005acc, 0x00010000, 32 },
    { 0x00005110, 0x04040404, 32 },
    { 0x00005114, 0x44444444, 32 },

    { 0x0000511c, 0x00b06cd8, 32 },

    { 0x00005120, 0x00008122, 32 },

    { 0x00005124, 0x019007c1, 32 },

    { 0x0000512c, 0x01, 8 },
    { 0x0000512d, 0x80, 8 },
    { 0x0000512f, 0x00, 8 },

    { 0x00005138, 0x00000000, 32 },

    { 0x0000513d, 0x00, 8 },
    { 0x0000513e, 0x33, 8 },
    { 0x0000513f, 0xa6, 8 },
    { 0x00005140, 0x24908804, 32 },
    { 0x00005145, 0x08, 8 },

    { 0x00005148, 0xc2ff3800, 32 },

    { 0x0000515c, 0x32828069, 32 },

    { 0x00005161, 0x40, 8 },
    { 0x00005164, 0x24, 8 },
    { 0x00005165, 0x60, 8 },

    { 0x0000516a, 0x18, 8 },

    { 0x00005171, 0x00, 8 },
    { 0x00005172, 0x05, 8 },

    { 0x00005176, 0x12, 8 },
    { 0x00005177, 0x40, 8 },

    { 0x00005181, 0x44, 8 },
    { 0x00005191, 0x00, 8 },
    { 0x00005203, 0x02, 8 },

    { 0x00005390, 0x6c6c6c6c, 32 },
    { 0x00005394, 0x6c6c6c6c, 32 },
    { 0x00005398, 0x6c6c6c6c, 32 },
    { 0x0000539c, 0x6c6c6c6c, 32 },
    { 0x000053a0, 0x6c6c6c6c, 32 },
    { 0x000053a4, 0x6c6c6c6c, 32 },
    { 0x000053a8, 0x6c6c6c6c, 32 },
    { 0x000053ac, 0x6c6c6c6c, 32 },
    { 0x000053b0, 0x6c6c6c6c, 32 },
    { 0x000053b4, 0x6c6c6c6c, 32 },
    { 0x000053b8, 0x6c6c6c6c, 32 },
    { 0x000053bc, 0x6c6c6c6c, 32 },
    { 0x000053c0, 0x6c6c6c6c, 32 },
    { 0x000053c4, 0x6c6c6c6c, 32 },
    { 0x000053c8, 0x6c6c6c6c, 32 },
    { 0x000053cc, 0x6c6c6c6c, 32 },
    { 0x000053d0, 0x6c6c6c6c, 32 },
    { 0x000053d4, 0x6c6c6c6c, 32 },


    { 0x00005a00, 0x48484848, 32 },
    { 0x00005a04, 0x48484848, 32 },
    { 0x00005a08, 0x48484848, 32 },
    { 0x00005a0c, 0x48484848, 32 },

    { 0x00005a10, 0x00004848, 32 },
    { 0x00005a14, 0x00000000, 32 },
    { 0x00005a18, 0x00000000, 32 },
    { 0x00005a1c, 0x00000000, 32 },
    { 0x00005a20, 0x00000000, 32 },

    { 0x00005a24, 0x6a6a6a6a, 32 },
    { 0x00005a28, 0x6a6a6a6a, 32 },
    { 0x00005a2c, 0x6a6a6a6a, 32 },
    { 0x00005a30, 0x6a6a6a6a, 32 },
    { 0x00005a34, 0x6c6c6a6a, 32 },
    { 0x00005a38, 0x6c6c6c6c, 32 },
    { 0x00005a3c, 0x6c6c6c6c, 32 },
    { 0x00005a40, 0x6c6c6c6c, 32 },
    { 0x00005a44, 0x6c6c6c6c, 32 },
    { 0x00005a48, 0x48484848, 32 },
    { 0x00005a4c, 0x48484848, 32 },
    { 0x00005a50, 0x48484848, 32 },
    { 0x00005a54, 0x48484848, 32 },

    { 0x00005a58, 0x00004848, 32 },
    { 0x00005a6c, 0x00000000, 32 },
    { 0x00005a70, 0x00000000, 32 },
    { 0x00005a74, 0x00000000, 32 },
    { 0x00005a78, 0x00000000, 32 },
    { 0x00005a7c, 0x00000000, 32 },
    { 0x00005a80, 0x00000000, 32 },
    { 0x00005a84, 0x00000000, 32 },
    { 0x00005a88, 0x00000000, 32 },
    { 0x00005a8c, 0x00000000, 32 },

    { 0x00005d5c, 0x2e2e0000, 32 },
    { 0x00005d60, 0x00000000, 32 },
    { 0x00005e0c, 0x00626200, 32 },
    { 0x00005e18, 0x04000000, 32 },
    { 0x00005e1c, 0x04040404, 32 },
    { 0x00005e20, 0x04040404, 32 },
    { 0x00005e24, 0x04040404, 32 },
    { 0x00005e28, 0x04040404, 32 },
    { 0x00005e2c, 0x04040404, 32 },
    { 0x00005e30, 0x04040404, 32 },
    { 0x00005e34, 0x04040404, 32 },
    { 0x00005e38, 0x04040404, 32 },
    { 0x00005e3c, 0x00040404, 32 },

    { 0x00005300, 0x00000000, 32 },
    { 0x00005304, 0x00000000, 32 },
    { 0x00005308, 0x00000000, 32 },
    { 0x0000530c, 0x00000000, 32 },
    { 0x00005310, 0x00000000, 32 },
    { 0x00005314, 0x00000000, 32 },
    { 0x00005318, 0x00000000, 32 },
    { 0x0000531c, 0x00000000, 32 },
    { 0x00005320, 0x00000000, 32 },
    { 0x00005324, 0x00000000, 32 },
    { 0x00005328, 0x00000000, 32 },
    { 0x0000532c, 0x00000000, 32 },
    { 0x00005330, 0x00000000, 32 },
    { 0x00005334, 0x00000000, 32 },
    { 0x00005338, 0x00000000, 32 },
    { 0x0000533c, 0x00000000, 32 },
    { 0x00005340, 0x00000000, 32 },
    { 0x00005344, 0x00000000, 32 },
    { 0x00005348, 0x00000000, 32 },
    { 0x0000534c, 0x00000000, 32 },
    { 0x00005350, 0x00000000, 32 },
    { 0x00005354, 0x00000000, 32 },
    { 0x00005358, 0x00000000, 32 },
    { 0x0000535c, 0x00000000, 32 },
    { 0x00005360, 0x00000000, 32 },
    { 0x00005364, 0x00000000, 32 },
    { 0x00005368, 0x00000000, 32 },
    { 0x0000536c, 0x00000000, 32 },
    { 0x00005370, 0x00000000, 32 },
    { 0x00005374, 0x00000000, 32 },
    { 0x00005378, 0x00000000, 32 },
    { 0x0000537c, 0x00000000, 32 },
    { 0x00005380, 0x00000000, 32 },
    { 0x00005384, 0x00000000, 32 },
    { 0x00005388, 0x00000000, 32 },
    { 0x0000538c, 0x00000000, 32 },

    { 0x000053d8, 0x00000000, 32 },
    { 0x000053dc, 0x00000000, 32 },
    { 0x000053e0, 0x00000000, 32 },
    { 0x000053e4, 0x00000000, 32 },
    { 0x000053e8, 0x00000000, 32 },
    { 0x000053ec, 0x00000000, 32 },
    { 0x000053f0, 0x00000000, 32 },
    { 0x000053f4, 0x00000000, 32 },
    { 0x000053f8, 0x00000000, 32 },
    { 0x000053fc, 0x00000000, 32 },
    { 0x0000513c, 0x02, 8 },
    { 0x0000513c, 0x06, 8 },
    { 0x0000591d, 0x0c, 8 },

    { 0x0000508c, 0x01, 8 },
    { 0x0000508c, 0x00, 8 },
    { 0x0000857c, 0x80000000, 32 },
    { 0x00005014, 0x80, 8 },
    { 0x0000500b, 0x00, 8 },
    { 0x00008af2, 0x01, 8 },
    { 0x00008aa0, 0x10, 8 }
};
#elif FPGA_DDR2
static MMIOREGISTER Mclk_300_Timing_Desktop[] = {
    { 0x0000516c, 0x10, 8 },
    { 0x00005040, 0x00000000, 32 },
    { 0x00005044, 0x00000000, 32 },
    { 0x00005100, 0x40302010, 32 },
    { 0x00005104, 0x80706050, 32 },
    { 0x00005108, 0x30201000, 32 },
    { 0x0000510c, 0x70605040, 32 },
    { 0x00005158, 0x67380202, 32 },
    { 0x0000515c, 0x1f83177f, 32 },
    { 0x00005160, 0x88008000, 32 },
    { 0x00005164, 0x00000000, 32 },
    { 0x00005168, 0x5d004e4e, 32 },
    { 0x0000516c, 0x48118030, 32 },
    { 0x00005250, 0x00ff0000, 32 },

    { 0x0000516c, 0x48118030, 32 },
    { 0x00005118, 0xba98ba98, 32 },
    { 0x0000511c, 0xd4, 8 },
    { 0x0000511d, 0x6c, 8 },
    { 0x0000511e, 0xb0, 8 },
    { 0x00005120, 0x01, 8 },
    { 0x00005121, 0x80, 8 },
    { 0x00005126, 0xc0, 8 },
    { 0x0000512c, 0x01, 8 },
    { 0x0000512d, 0x80, 8 },
    { 0x0000512f, 0x00, 8 },
    { 0x00005138, 0x00000000, 32 },
    { 0x0000513d, 0x00, 8 },
    { 0x0000513e, 0x33, 8 },
    { 0x0000513f, 0xa6, 8 },
    { 0x00005140, 0x26108804, 32 },
    { 0x00005145, 0x08, 8 },
    { 0x00005148, 0xc2ff2800, 32 },
    { 0x0000515c, 0x1f83177f, 32 },
    { 0x00005161, 0x80, 8 },
    { 0x00005164, 0x00, 8 },
    { 0x00005165, 0x00, 8 },
    { 0x0000516a, 0x00, 8 },
    { 0x00005170, 0x40, 8 },
    { 0x00005171, 0x00, 8 },
    { 0x00005172, 0x15, 8 },
    { 0x00005176, 0x12, 8 },
    { 0x00005177, 0x40, 8 },
    { 0x00005181, 0x44, 8 },
    { 0x00005191, 0x00, 8 },
    { 0x00005203, 0x02, 8 },
    { 0x00005390, 0x90909090, 32 },
    { 0x00005394, 0x90909090, 32 },
    { 0x00005398, 0x90909090, 32 },
    { 0x0000539c, 0x90909090, 32 },
    { 0x000053a0, 0x90909090, 32 },
    { 0x000053a4, 0x90909090, 32 },
    { 0x000053a8, 0x90909090, 32 },
    { 0x000053ac, 0x90909090, 32 },
    { 0x000053b0, 0x90909090, 32 },
    { 0x000053b4, 0x90909090, 32 },
    { 0x000053b8, 0x90909090, 32 },
    { 0x000053bc, 0x90909090, 32 },
    { 0x000053c0, 0x90909090, 32 },
    { 0x000053c4, 0x90909090, 32 },
    { 0x000053c8, 0x90909090, 32 },
    { 0x000053cc, 0x90909090, 32 },
    { 0x000053d0, 0x90909090, 32 },
    { 0x000053d4, 0x90909090, 32 },
    { 0x00005a00, 0x48484848, 32 },
    { 0x00005a04, 0x48484848, 32 },
    { 0x00005a08, 0x48484848, 32 },
    { 0x00005a0c, 0x48484848, 32 },
    { 0x00005a10, 0x00004848, 32 },
    { 0x00005a24, 0x6a6a6a6a, 32 },
    { 0x00005a28, 0x6a6a6a6a, 32 },
    { 0x00005a2c, 0x6a6a6a6a, 32 },
    { 0x00005a30, 0x6a6a6a6a, 32 },
    { 0x00005a34, 0x6c6c6a6a, 32 },
    { 0x00005a38, 0x6c6c6c6c, 32 },
    { 0x00005a3c, 0x6c6c6c6c, 32 },
    { 0x00005a40, 0x6c6c6c6c, 32 },
    { 0x00005a44, 0x6c6c6c6c, 32 },
    { 0x00005a48, 0x48484848, 32 },
    { 0x00005a4c, 0x48484848, 32 },
    { 0x00005a50, 0x48484848, 32 },
    { 0x00005a54, 0x48484848, 32 },
    { 0x00005a58, 0x00004848, 32 },
    { 0x00005a6c, 0x00000000, 32 },
    { 0x00005a70, 0x00000000, 32 },
    { 0x00005a74, 0x00000000, 32 },
    { 0x00005a78, 0x00000000, 32 },
    { 0x00005a7c, 0x00000000, 32 },
    { 0x00005a80, 0x00000000, 32 },
    { 0x00005a84, 0x00000000, 32 },
    { 0x00005a88, 0x00000000, 32 },
    { 0x00005a8c, 0x00000000, 32 },
    { 0x00005d5c, 0x2e2e0000, 32 },
    { 0x00005d60, 0x00000000, 32 },
    { 0x00005e0c, 0x00626200, 32 },
    { 0x00005e18, 0x04000000, 32 },
    { 0x00005e1c, 0x04040404, 32 },
    { 0x00005e20, 0x04040404, 32 },
    { 0x00005e24, 0x04040404, 32 },
    { 0x00005e28, 0x04040404, 32 },
    { 0x00005e2c, 0x04040404, 32 },
    { 0x00005e30, 0x04040404, 32 },
    { 0x00005e34, 0x04040404, 32 },
    { 0x00005e38, 0x04040404, 32 },
    { 0x00005e3c, 0x00040404, 32 },
    { 0x00005300, 0x01010101, 32 },
    { 0x00005304, 0x01010101, 32 },
    { 0x00005308, 0x01010101, 32 },
    { 0x0000530c, 0x01010101, 32 },
    { 0x00005310, 0x01010101, 32 },
    { 0x00005314, 0x01010101, 32 },
    { 0x00005318, 0x01010101, 32 },
    { 0x0000531c, 0x01010101, 32 },
    { 0x00005320, 0x01010101, 32 },
    { 0x00005324, 0x01010101, 32 },
    { 0x00005328, 0x01010101, 32 },
    { 0x0000532c, 0x01010101, 32 },
    { 0x00005330, 0x01010101, 32 },
    { 0x00005334, 0x01010101, 32 },
    { 0x00005338, 0x01010101, 32 },
    { 0x0000533c, 0x01010101, 32 },
    { 0x00005340, 0x01010101, 32 },
    { 0x00005344, 0x01010101, 32 },
    { 0x00005348, 0x01010101, 32 },
    { 0x0000534c, 0x01010101, 32 },
    { 0x00005350, 0x01010101, 32 },
    { 0x00005354, 0x01010101, 32 },
    { 0x00005358, 0x01010101, 32 },
    { 0x0000535c, 0x01010101, 32 },
    { 0x00005360, 0x01010101, 32 },
    { 0x00005364, 0x01010101, 32 },
    { 0x00005368, 0x01010101, 32 },
    { 0x0000536c, 0x01010101, 32 },
    { 0x00005370, 0x01010101, 32 },
    { 0x00005374, 0x01010101, 32 },
    { 0x00005378, 0x01010101, 32 },
    { 0x0000537c, 0x01010101, 32 },
    { 0x00005380, 0x01010101, 32 },
    { 0x00005384, 0x01010101, 32 },
    { 0x00005388, 0x01010101, 32 },
    { 0x0000538c, 0x01010101, 32 },
    { 0x000053d8, 0x55555555, 32 },
    { 0x000053dc, 0x55555550, 32 },
    { 0x000053e0, 0x55555055, 32 },
    { 0x000053e4, 0x55505555, 32 },
    { 0x000053e8, 0x50555555, 32 },
    { 0x000053ec, 0x55555555, 32 },
    { 0x000053f0, 0x55555550, 32 },
    { 0x000053f4, 0x55555055, 32 },
    { 0x000053f8, 0x55505555, 32 },
    { 0x000053fc, 0x50555555, 32 },

    { 0x0000513c, 0x02, 8 },

    { 0x0000513c, 0x06, 8 },
    { 0x0000508e, 0x01, 8 },

    { 0x0000508e, 0x00, 8 },
    { 0x000490a0, 0x1fffff1f, 32 },
    { 0x0000857C, 0x80000000, 32 },

    { 0x00005017, 0x80, 8 },
    { 0x0000500b, 0x00, 8 },
    { 0x00008af1, 0x01, 8 },
    { 0x0000850c, 0x02, 8 },
    { 0x00008aa0, 0x10, 8 },
    { 0x00005158, 0x67380204, 32 },
};
#else //not QT
static CBREGISTER Mclk_300_Timing_Desktop[]={
};

#endif

static CBREGISTER SRDefaultVCP[]=
{
    {SR, (CBIOS_U8)~0xFE,0x09,0x7E},   
    {SR, (CBIOS_U8)~0x8D,0x0B,0x00},  
    {SR, (CBIOS_U8)~0xF7,0x0D,0x50},
    {SR, (CBIOS_U8)~0xFF, 0x0E, 0xC1},
    {SR, (CBIOS_U8)~0xFF, 0x0F, 0x6B},
    {SR, (CBIOS_U8)~0xDF, 0x14, 0x00},
    {SR, (CBIOS_U8)~0x6E, 0x15, 0x4A},
    {SR, (CBIOS_U8)~0x20, 0x18,0x20},
    {SR, (CBIOS_U8)~0xFF, 0x19, 0x31},
    {SR, (CBIOS_U8)~0x1F, 0x1C, 0x1C},
    {SR, (CBIOS_U8)~0x67, 0x1D, 0x67},
    {SR, (CBIOS_U8)~0x1F, 0x1E, 0x12},
    {SR, (CBIOS_U8)~0xF1, 0x1F, 0x50},
    {SR, (CBIOS_U8)~0xFF, 0x20, 0x7F},
    {SR, (CBIOS_U8)~0xFF, 0x21, 0xCE},
    {SR, (CBIOS_U8)~0xE7, 0x10, 0x84},
    {SR, (CBIOS_U8)~0x7F, 0x22, 0x0E},
    {SR, (CBIOS_U8)~0xFF, 0x23, 0xE8},
    {SR, (CBIOS_U8)~0x3F, 0x24, 0x10},
    {SR, (CBIOS_U8)~0xFF, 0x25, 0xC8},
    {SR, (CBIOS_U8)~0x80, 0x26, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x27, 0xB3},
    {SR, (CBIOS_U8)~0x3C, 0x28, 0x14},
    {SR, (CBIOS_U8)~0xFF, 0x29, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x2B, 0x00},
    {SR, (CBIOS_U8)~0xD0, 0x2D, 0x00},
    {SR, (CBIOS_U8)~0x3F, 0x30, 0x03},
    {SR, (CBIOS_U8)~0x76, 0x31, 0x00},
    {SR, (CBIOS_U8)~0xF8, 0x39, 0x98},
    {SR, (CBIOS_U8)~0x18, 0x3C, 0x18},
    {SR, (CBIOS_U8)~0x3F, 0x3F, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x42, 0x01},
    {SR, (CBIOS_U8)~0xFF, 0x43, 0x01},
    {SR, (CBIOS_U8)~0x88, 0x44, 0x80},
    {SR, (CBIOS_U8)~0x9F, 0x45, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x47, 0x01},
    {SR, (CBIOS_U8)~0xFF, 0x4F, 0x00},
    {SR, (CBIOS_U8)~0x01, 0x70, 0x00},
    {SR, (CBIOS_U8)~0x10, 0x94, 0x00},
    {SR, (CBIOS_U8)~0x10, 0x40, 0x00},
    {SR, (CBIOS_U8)~0x81, 0x37, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x38, 0x00},
    {SR, (CBIOS_U8)~0xFF, 0x3E, 0x00},
    {SR, (CBIOS_U8)~0x1C, 0x31, 0x1C},
    {SR, (CBIOS_U8)~0xFF, 0x36, 0x00},//disable dither for dp1
    {SR_B, (CBIOS_U8)~0xFF, 0x36, 0x00},//disable dither for dp2
};

static CBREGISTER CRDefaultVCP[]=
{
    {CR, (CBIOS_U8)~0x09, 0x31, 0x09},
    {CR, (CBIOS_U8)~0xB0, 0x34, 0x00},
    {CR, (CBIOS_U8)~0x01, 0x3D, 0x00},
    {CR, (CBIOS_U8)~0x68, 0x43, 0x00},
    {CR, (CBIOS_U8)~0x0F, 0x45, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0x52, 0x00},
    {CR, (CBIOS_U8)~0x20, 0x55, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0x59, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0x5A, 0x00},
    {CR, (CBIOS_U8)~0xF8, 0x5B, 0x00},
    {CR, (CBIOS_U8)~0xD9, 0x63, 0x00},
    {CR, (CBIOS_U8)~0x02, 0x66, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0x6B, 0x01},
    {CR, (CBIOS_U8)~0xFF, 0x6C, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0x6D, 0x33},
    {CR, (CBIOS_U8)~0xFF, 0x6E, 0x00},
    {CR, (CBIOS_U8)~0x22, 0x71, 0x22},
    {CR, (CBIOS_U8)~0xFF, 0x86, 0xA8},
    {CR, (CBIOS_U8)~0xFF, 0x87, 0x68},
    {CR, (CBIOS_U8)~0xFF, 0x88, 0x20},
    {CR, (CBIOS_U8)~0xFF, 0x89, 0xE0},
    {CR, (CBIOS_U8)~0xFF, 0x8D, 0x68},
    {CR, (CBIOS_U8)~0xFF, 0xA1, 0x00},
    {CR, (CBIOS_U8)~0xFB, 0xA2, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xA3, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xA4, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xA6, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xB0, 0x55},
    {CR, (CBIOS_U8)~0x1F, 0xB1, 0x09},
    {CR, (CBIOS_U8)~0x3F, 0xB2, 0x01},
    {CR, (CBIOS_U8)~0x3F, 0xB3, 0x01},
    {CR, (CBIOS_U8)~0xFF, 0xBC, 0x20},
    {CR, (CBIOS_U8)~0xFF, 0xBD, 0xA8},
    {CR, (CBIOS_U8)~0xFF, 0xBE, 0x08},
    {CR, (CBIOS_U8)~0xFF, 0xC0, 0x24},
    {CR, (CBIOS_U8)~0xFF, 0xC1, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC2, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC3, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC4, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC5, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC6, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC7, 0x00},
    {CR, (CBIOS_U8)~0xFF, 0xC8, 0x00},
    {CR, (CBIOS_U8)~0x01, 0xCD, 0x00},
    {CR, (CBIOS_U8)~0x1F, 0xCE, 0x0F},
    {CR, (CBIOS_U8)~0x80, 0xD4, 0x80},
    {CR_B, (CBIOS_U8)~0x09, 0x31, 0x09},
    {CR_B, (CBIOS_U8)~0x22, 0x71, 0x22},
    {CR_B, (CBIOS_U8)~0x1F, 0xB1, 0x09},
    {CR_B, (CBIOS_U8)~0xFF, 0xC0, 0x7F},
    {CR_B, (CBIOS_U8)~0xFF, 0xC1, 0x7F},
    {CR_B, (CBIOS_U8)~0xFF, 0xC2, 0x00},
    {CR_B, (CBIOS_U8)~0xFF, 0xC3, 0x00},
    {CR_B, (CBIOS_U8)~0xFF, 0xC4, 0x00},
    {CR_B, (CBIOS_U8)~0x1F, 0xCE, 0x0F},
    {CR_B, (CBIOS_U8)~0x40, 0xD2, 0x40},
    {CR_B, (CBIOS_U8)~0x1F, 0xE0, 0x04},
    {CR_B, (CBIOS_U8)~0xFF, 0xF5, 0x09},
    {CR_B, (CBIOS_U8)~0xFF, 0xF6, 0x1F},
    {CR_B, (CBIOS_U8)~0xFF, 0xFC, 0x00},
    {CR_B, (CBIOS_U8)~0x80, 0xFD, 0x00},
    {CR_C, (CBIOS_U8)~0x04, 0x9D, 0x04},
    {CR_C, (CBIOS_U8)~0x1F, 0xA0, 0x02},
    {CR_C, (CBIOS_U8)~0x10, 0xA1, 0x00},
    {CR_C,  (CBIOS_U8)~0x01, 0xA2,0x00},
    {CR_C, (CBIOS_U8)~0xC0, 0xA3, 0x00},
    {CR_C, (CBIOS_U8)~0xA0, 0xA4, 0x00},
    {CR_C, (CBIOS_U8)~0x80, 0xA5, 0x00},
    {CR_C, (CBIOS_U8)~0xD0, 0xA6, 0x00},
    {CR_C, (CBIOS_U8)~0x3E, 0xC2, 0x00},
};

static CBREGISTER ZXGModeExtRegDefault[] =
{
    {SR,(CBIOS_U8)~0xC0, 0x19, 0x00},
    {SR,(CBIOS_U8)~0x60, 0x1A, 0x00},
    {SR,(CBIOS_U8)~0xF8, 0x1B, 0x00},
    {SR,(CBIOS_U8)~0xD8, 0x47, 0x00},   
    {CR,(CBIOS_U8)~0xEF, 0x31, 0x05},
    {CR,(CBIOS_U8)~0x44, 0x32, 0x40},
    {CR,(CBIOS_U8)~0x7A, 0x33, 0x08},
    {CR,(CBIOS_U8)~0x38, 0x3A, 0x00},
    {CR, 0x00, 0x3B, 0x00},
    {CR, 0x00, 0x3C, 0x10},
    {CR,(CBIOS_U8)~0x3F, 0x42, 0x00},   
    {CR,(CBIOS_U8)~0xF1, 0x50, 0x00},
    {CR,(CBIOS_U8)~0x7F, 0x51, 0x00},
    {CR,(CBIOS_U8)~0x80, 0x53, 0x00},
    {CR, 0x00, 0x5D, 0x00},
    {CR, 0x00, 0x5E, 0xC0},	 
    {CR,(CBIOS_U8)~0x0F, 0x64, 0x00},
    {CR,(CBIOS_U8)~0xFB, 0x65, 0x00},
    {CR,(CBIOS_U8)~0x01, 0x66, 0x00},
    {CR, 0x00, 0x69, 0x00},
    {CR, 0x00, 0x6A, 0x00},
};

static CBREGISTER ZXGMode3SRRegTable[] =
{
    {SR,     0x00, 0x02, 0x03},
    {SR,     0x00, 0x03, 0x00},
    {SR,     0x00, 0x04, 0x02},
};

static CBREGISTER ZXGMode3CRRegTable[] =
{
    {CR,     0x00, 0x00, 0x5F},
    {CR,     0x00, 0x01, 0x4F},
    {CR,     0x00, 0x02, 0x50},
    {CR,     0x00, 0x03, 0x82},
    {CR,     0x00, 0x04, 0x55},
    {CR,     0x00, 0x05, 0x81},
    {CR,     0x00, 0x06, 0xBF},
    {CR,     0x00, 0x07, 0x1F},
    {CR,    ~0x7F, 0x08, 0x00},
    {CR,     0x00, 0x09, 0x4F},
    {CR,    ~0x3F, 0x0A, 0x0D},
    {CR,    ~0x7F, 0x0B, 0x0E},
    {CR,     0x00, 0x0C, 0x00},
    {CR,     0x00, 0x0D, 0x00},
    {CR,     0x00, 0x0E, 0x00},
    {CR,     0x00, 0x0F, 0x00},
    {CR,     0x00, 0x10, 0x9C},
    {CR,     0x00, 0x11, 0x8E},
    {CR,     0x00, 0x12, 0x8F},
    {CR,     0x00, 0x13, 0x28},
    {CR,    ~0x7F, 0x14, 0x1F},
    {CR,     0x00, 0x15, 0x96},
    {CR,     0x00, 0x16, 0xB9},
    {CR,     0x00, 0x17, 0xA3},
    {CR,     0x00, 0x18, 0xFF},
    {CR,    ~0x3F, 0x42, 0x00},
    {CR,    ~0x7F, 0x51, 0x00},
    {CR,    ~0x7F, 0x5D, 0x00},
    {CR,     0x00, 0x5E, 0xC0},	 
    {CR,     0x00, 0x6A, 0x00},
};

static CBREGISTER ZXGMode3ARRegTable[] =
{
    {AR,  0x00, 0x00, 0x00},
    {AR,  0x00, 0x01, 0x01},
    {AR,  0x00, 0x02, 0x02},
    {AR,  0x00, 0x03, 0x03},
    {AR,  0x00, 0x04, 0x04},
    {AR,  0x00, 0x05, 0x05},
    {AR,  0x00, 0x06, 0x14},
    {AR,  0x00, 0x07, 0x07},
    {AR,  0x00, 0x08, 0x38},
    {AR,  0x00, 0x09, 0x39},
    {AR,  0x00, 0x0A, 0x3A},
    {AR,  0x00, 0x0B, 0x3B},
    {AR,  0x00, 0x0C, 0x3C},
    {AR,  0x00, 0x0D, 0x3D},
    {AR,  0x00, 0x0E, 0x3E},
    {AR,  0x00, 0x0F, 0x3F}, 

    {AR,  0x00, 0x10, 0x0C},
    {AR,  0x00, 0x11, 0x00},
    {AR,  0x00, 0x12, 0x0F},
    {AR,  0x00, 0x13, 0x08},
    {AR,  0x00, 0x14, 0x00},

    {GR,  0x00, 0x00, 0x00},
    {GR,  0x00, 0x01, 0x00},
    {GR,  0x00, 0x02, 0x00},
    {GR,  0x00, 0x03, 0x00},
    {GR,  0x00, 0x04, 0x00},
    {GR,  0x00, 0x05, 0x10},
    {GR,  0x00, 0x06, 0x0E},
    {GR,  0x00, 0x07, 0x00},
    {GR,  0x00, 0x08, 0xFF},
};

static CBREGISTER ZXGMode3RegOnIGA2[] =
{
    {CR, 0x00, 0x00, 0x5F},
    {CR, 0x00, 0x01, 0x4F},
    {CR, 0x00, 0x02, 0x50},
    {CR, 0x00, 0x03, 0x82},
    {CR, 0x00, 0x04, 0x52},
    {CR, 0x00, 0x05, 0x9E},
    {CR, 0x00, 0x06, 0x0B},
    {CR, 0x00, 0x07, 0x3E},
    {CR, 0x00, 0x08, 0x00},
    {CR, 0x00, 0x09, 0x40},
    {CR, 0x00, 0x0D, 0x00},    
    {CR, 0x00, 0x10, 0xEA},
    {CR, 0x00, 0x11, 0x0C},
    {CR, 0x00, 0x12, 0xDF},
    {CR, 0x00, 0x15, 0xE7},
    {CR, 0x00, 0x16, 0x05},
    {CR,(CBIOS_U8)~0x80, 0x17, 0x80},
    {CR, 0x00, 0x5D, 0x00},
    {CR, 0x00, 0x5E, 0xC0},	
    {CR,(CBIOS_U8)~0xC0, 0x63, 0x00},
};

static  CBIOS_U32 StreamThreshold[CBIOS_IGACOUNTS][STREAM_NUM_E3K-1] = 
{
    {0x8454, 0x3346c, 0x33428},
    {0x33928,0x33a9c, 0x33b18},
    {0x34028,0x3419c, 0x34218},
    {0x34728,0x3489c, 0x34918},
};

static  CBIOS_U32 StreamTimeout[CBIOS_IGACOUNTS] = 
{0x3328c, 0x332b0, 0x332b4, 0x332b8};

static  CBIOS_U32 HpdInvert[CBIOS_IGACOUNTS] = 
{0x334e0, 0x33d54, 0x34454, 0x34b54};

/*****************************************************************************/
/* ------------------------------------- HW Init --------------------------------------- */
/*****************************************************************************/

static CBIOS_STATUS cbChipEnable(PCBIOS_EXTENSION_COMMON pcbe)
{ 
    CBIOS_STATUS status = CBIOS_OK;

    // 850C bit 1 should be used to wake chip.
    cb_WriteU8(pcbe->pAdapterContext, 0x850C, 0x2);
    
    //Wakeup chip, 3C3 bit0 = 1
    cb_WriteU8(pcbe->pAdapterContext, 0x83C3, 0x01);

    // 3C2 MISCELLANEOUS reg
    // Bit 0 Color emulation. Address base at 0x3Dx.
    // Bit 1 Enable access of CPU
    cb_WriteU8(pcbe->pAdapterContext, 0x83C2, 0x03);

    cbUnlockCR(pcbe);
    cbUnlockSR(pcbe);

    cbMMIOWriteReg(pcbe, SR_2A, 0x00,  0x00);
    cbMMIOWriteReg(pcbe, SR_26, 0x40,  0x00);  

#if CBIOS_CHECK_HARDWARE_STATUS
    // If MMIO failure, try to use standard IO.
    if (cbIsMMIOWell(pcbe) == CBIOS_FALSE)
    {
        cbDebugPrint((MAKE_LEVEL(GENERIC, ERROR), "\n\nMMIO not enabled!!!\n"));

        return CBIOS_ER_MMIO;
    }

#endif


    return status;
}

#if 0
static CBIOS_VOID cbPOSTSetClock(PCBIOS_EXTENSION_COMMON pcbe)
{
    CBIOS_U32 dwEClock, dwVppClock;

    cbMMIOWriteReg(pcbe, SR_01, 0x20, 0xDF); //screen off

    dwEClock = pcbe->SysBiosInfo.ECLK;
    dwVppClock = pcbe->SysBiosInfo.VCLK;

    if(dwEClock < CBIOS_CLOCK_MIN || dwEClock > CBIOS_CLOCK_MAX)
    {
        dwEClock = CBIOS_ECLK_DEFAULT;
    }

     if(dwVppClock < CBIOS_CLOCK_MIN || dwVppClock > CBIOS_CLOCK_MAX)
    {
        dwVppClock = CBIOS_VCLK_DEFAULT;
    }
     
    dwEClock *= 10000;
    dwVppClock *= 10000;

    cbProgClock(pcbe,dwEClock, ECLKTYPE_Post, IGA1);
    cbProgClock(pcbe,dwVppClock, CBIOS_VCLKTYPE, IGA1);
}
#endif

static CBIOS_STATUS cbLoadDefaultTables(PCBIOS_EXTENSION_COMMON pcbe)
{   
    CBIOS_STATUS status = CBIOS_OK;

    //For QT only, load default MIU table.
    if (pcbe->DriverFlags.bRunOnQT && !pcbe->DriverFlags.bDriverLoadQTiming)
    {
        cbLoadMemoryTimingTbl(pcbe, Mclk_300_Timing_Desktop,
            sizeofarray(Mclk_300_Timing_Desktop));

        cbLoadtable(pcbe,
            SRDefaultVCP,
            sizeofarray(SRDefaultVCP),
            CBIOS_NOIGAENCODERINDEX);

        cbLoadtable(pcbe,
            CRDefaultVCP,
            sizeofarray(CRDefaultVCP),
            CBIOS_NOIGAENCODERINDEX);
    }

    cbLoadtable(pcbe, 
        pcbe->PostRegTable[VCP_TABLE].pSRDefault, 
        pcbe->PostRegTable[VCP_TABLE].sizeofSRDefault, 
        CBIOS_NOIGAENCODERINDEX);

    cbLoadtable(pcbe, 
        pcbe->PostRegTable[VCP_TABLE].pCRDefault, 
        pcbe->PostRegTable[VCP_TABLE].sizeofCRDefault, 
        CBIOS_NOIGAENCODERINDEX);

    return status;
}

static CBIOS_VOID cbEndOfPost(PCBIOS_EXTENSION_COMMON pcbe)
{ 

    REG_CR6B    RegCR6BValue;
    REG_CR6B    RegCR6BMask;
    REG_CR6C    RegCR6CValue;
    REG_CR6C    RegCR6CMask;

    // Clear active device in scratch pad.
    RegCR6BValue.Value = 0;
    RegCR6BValue.BIOS_Flag_3 = 0;
    RegCR6BMask.Value = 0xFF;
    RegCR6BMask.BIOS_Flag_3 = 0;
    cbMMIOWriteReg(pcbe,CR_6B, RegCR6BValue.Value, RegCR6BMask.Value);
    RegCR6CValue.Value = 0;
    RegCR6CValue.BIOS_Flag_4 = 0;
    RegCR6CMask.Value = 0xFF;
    RegCR6CMask.BIOS_Flag_4 = 0;
    cbMMIOWriteReg(pcbe,CR_6C, RegCR6CValue.Value, RegCR6CMask.Value);    

    cbUnlockSR(pcbe);
    cbUnlockCR(pcbe);
}

static CBIOS_VOID  cbSetupVGA(PCBIOS_EXTENSION_COMMON pcbe)
{
    cb_WriteU8(pcbe->pAdapterContext, CB_CRT_ADDR_REG, 0x41);
    cb_WriteU8(pcbe->pAdapterContext, CB_CRT_DATA_REG, 0x03);  //mode 3

    cbLoadtable(pcbe, ZXGModeExtRegDefault,
                sizeofarray(ZXGModeExtRegDefault),
                CBIOS_NOIGAENCODERINDEX);
    // do sync reset
    cb_WriteU16(pcbe->pAdapterContext, CB_SEQ_ADDR_REG, 0x0100);
    cb_WriteU16(pcbe->pAdapterContext, CB_SEQ_ADDR_REG, 0x2001);
    cb_WriteU16(pcbe->pAdapterContext, CB_SEQ_ADDR_REG, 0x2001);   // Do this two times

    cb_WriteU8(pcbe->pAdapterContext, CB_MISC_OUTPUT_WRITE_REG,0x67);
    cb_WriteU16(pcbe->pAdapterContext, CB_SEQ_ADDR_REG, 0x0300);

    cb_DelayMicroSeconds( 150);

    cbLoadtable(pcbe,
                ZXGMode3SRRegTable,
                sizeofarray(ZXGMode3SRRegTable),
                CBIOS_NOIGAENCODERINDEX);
    cb_WriteU16(pcbe->pAdapterContext, CB_CRT_ADDR_REG, 0x0011);  // Unlock writing to CRT

    cbLoadtable(pcbe,
                ZXGMode3CRRegTable,
                sizeofarray(ZXGMode3CRRegTable),
                CBIOS_NOIGAENCODERINDEX);
    cbLoadtable(pcbe,
                ZXGMode3RegOnIGA2,
                sizeofarray(ZXGMode3RegOnIGA2),
                IGA2);

    cbLoadtable(pcbe,
                ZXGMode3RegOnIGA2,
                sizeofarray(ZXGMode3RegOnIGA2),
                IGA3);

    cbLoadtable(pcbe,
                ZXGMode3RegOnIGA2,
                sizeofarray(ZXGMode3RegOnIGA2),
                IGA4);


    cb_ReadU8(pcbe->pAdapterContext, CB_INPUT_STATUS_1_REG);  // lock writing to CRT

    cbLoadtable(pcbe,
                ZXGMode3ARRegTable,
                sizeofarray(ZXGMode3ARRegTable),
                CBIOS_NOIGAENCODERINDEX);
}

CBIOS_STATUS cbPost(PCBIOS_VOID pvcbe)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_STATUS    status = CBIOS_OK;
    REG_CR68    RegCR68Value, RegCR68Mask;

    /*this is a patch, we must write something to 3c6h,3c7h,3c8h or 3c9h 
    to generate hblank vblank signal for CRT*/
    cb_WriteU8(pcbe->pAdapterContext, 0x83C8, 0);
    
    cbWritePort80(pcbe,CBIOS_PORT80_ID_CBiosPost_Enter);

    status = cbChipEnable(pcbe);
    cbWritePort80(pcbe, CBIOS_PORT80_ID_AfterChipEnable);
    if( status != CBIOS_OK)
    {
        cbDebugPrint((MAKE_LEVEL(GENERIC, ERROR),"Chip is not enable!\n"));
        cbPrintMsgToFile(DBG_LEVEL_ERROR_MSG, "cbPost: Chip is not enable!\n", CBIOS_NULL, 0);
        return status;  
    }

    RegCR68Value.Value = 0;
    RegCR68Value.EOCLK_Source = 0;
    RegCR68Mask.Value = 0xFF;
    RegCR68Mask.EOCLK_Source = 0;
    cbMMIOWriteReg(pcbe, CR_68, RegCR68Value.Value, RegCR68Mask.Value);       //force internal Eclk

    cbMMIOWriteReg(pcbe, CR_C_F3, 0x04, ~0x04); //reset will not check PLL lock
    //Just keep default clock now and GFX will adjust it dynamically
    if(pcbe->ChipID >= CHIPID_CHX004)
    {
        //cbPOSTSetClock(pcbe);
    }
    cbWritePort80(pcbe,CBIOS_PORT80_ID_AfterPrePost);
    
    cbLoadDefaultTables(pcbe);   
    cbWritePort80(pcbe,CBIOS_PORT80_ID_AfterLoadDefautTables);

    cbWritePort80(pcbe,CBIOS_PORT80_ID_AfterInitMemory);   

    cbWritePort80(pcbe,CBIOS_PORT80_ID_AfterIsMemoryWriteable);

    if(!pcbe->DriverFlags.bRunOnQT)
    {
        cbSetupVGA(pcbe);
    }

    // Per Henry, this code should come AFTER the MIU reset
    cbEndOfPost(pcbe);
    
    cbWritePort80(pcbe,CBIOS_PORT80_ID_CBiosPost_Exit);
    
    pcbe->DbgDataToFile[0] = status;
    cbPrintMsgToFile(DBG_LEVEL_CHIP_INFO, "Exit cbPost and return status", pcbe->DbgDataToFile, 4);

    cbHWSetChipPostFlag(pcbe);

    return status;
}

static CBIOS_VOID cbGetPMPInfo(PCBIOS_VOID pvcbe)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_UCHAR     PMPVer[E3K_PMPVER_SIZE] = {0};
    CBIOS_UCHAR     PMPTag[E3K_PMPTAG_SIZE] = {0};
    CBIOS_U8    DateIndex = 0, TimeIndex = 0;
    CBIOS_U8    Count = 0;
    CBIOS_U8 i = 0;

    for(i = 0; i < E3K_PMPTAG_SIZE; i++)
    {
        PMPTag[i] = cb_ReadU8(pcbe->pAdapterContext, E3K_PMPTAG_OFFSET+i);
    }

    if(!cbStrnCmp(PMPTag, (CBIOS_UCHAR*)"pmp", 3))
    {
        for(i = 0; i < E3K_PMPVER_SIZE; i++)
        {
            PMPVer[i] = cb_ReadU8(pcbe->pAdapterContext, E3K_PMPVER_OFFSET+i);
            if(PMPVer[i] == '\0' && Count < 2)
            {
                if(Count == 0)
                {
                    DateIndex = i+1;
                }
                else
                {
                    TimeIndex = i+1;
                }
                Count++;
            }
        }
        cb_memcpy(pcbe->PMPVer, PMPVer, E3K_PMPVER_SIZE);
        cbDebugPrint((MAKE_LEVEL(GENERIC, INFO),"VBIOS sub version:%s Build Time: %s %s \n", PMPVer, PMPVer+DateIndex, PMPVer+TimeIndex));
    }
    else
    {
        cbDebugPrint((MAKE_LEVEL(GENERIC, INFO),"Can't get VBIOS sub version\n"));
    }
}

CBIOS_STATUS cbInitChip(PCBIOS_VOID pvcbe)
{
    PCBIOS_EXTENSION_COMMON pcbe = (PCBIOS_EXTENSION_COMMON)pvcbe;
    CBIOS_U32       SupportDevices = 0, Index = 0, IndexStrm = 0, CurDev = 0;
    PCBIOS_DEVICE_COMMON    pDevCommon = CBIOS_NULL;
    CBIOS_GAMMA_PARA   GammaSet = {0};

    if(CHIPID_E3K == pcbe->ChipID || pcbe->ChipID == CHIPID_CHX004 || pcbe->ChipID == CHIPID_CNE001)
    {
        cbGetPMPInfo(pcbe);
    }

    SupportDevices = pcbe->DeviceMgr.SupportDevices;
    while (SupportDevices)
    {
        CurDev = GET_LAST_BIT(SupportDevices);
        pDevCommon = cbGetDeviceCommon(&pcbe->DeviceMgr, CurDev);

        cbDevDeviceHwInit(pcbe, pDevCommon);
        SupportDevices &= (~CurDev);
    }

    cbDIU_HDAC_SetStatus(pcbe);

    GammaSet.Flags.bDisableGamma = 1;
    for(Index = 0; Index < pcbe->DispMgr.IgaCount; Index++)
    {
        //patch for Issue 137273: if Dac color mode = 4bpp/8bpp, Ps will go though LUT even gamma is disabled!
        cbBiosMMIOWriteReg(pcbe, CR_67, 0x40, ~0x70, Index);
        //patch for Issue137093, shut down screen to prevent garbage.
        cbBiosMMIOWriteReg(pcbe, CR_71, 0x20, ~0x20, Index);
        //disable stream
        cbDisableStream(pcbe, Index);

        //disable gamma
        GammaSet.IGAIndex = Index;
        cbSetGamma(pcbe, &GammaSet);
    }

    if(CHIPID_E3K == pcbe->ChipID || pcbe->ChipID == CHIPID_CHX004 || pcbe->ChipID == CHIPID_CNE001)
    {
        cbMMIOWriteReg32(pcbe, 0x33524, 0xfffff, ~0xfffff); // enable all security_r of DIU
    }

    //diu idle gate enable
    if(pcbe->ChipID >= CHIPID_CHX004)
    {
        //default gate on DIU_IDLE(no memory access), gate off VBLANK and UMARDY4DFS
        //and set HDAUDIO_CARE
        //change set HDAUDIO_CARE to 0. The audio idle signal will consider CTL0/CTL1/RORB/RIRB,
        //but the audio controller will set CORB/RIRB to 1 when enter OS. Also the dclk/stream/device
        //are all off when long idle, the audio data can't send at all.
        cbMMIOWriteReg32(pcbe, 0x81b0, 0x020000, ~0x2E0000);
    }

    if((CHIPID_E3K == pcbe->ChipID || pcbe->ChipID == CHIPID_CHX004 || pcbe->ChipID == CHIPID_CNE001)
       && !pcbe->DriverFlags.bRunOnQT)
    {
       for(Index = 0; Index < pcbe->DispMgr.IgaCount; Index++)
       {
           //threshold, PS and SS share the same register
           for(IndexStrm = 0; IndexStrm < pcbe->DispMgr.StreamCount[Index]-1; IndexStrm++)
           {
               cbMMIOWriteReg32(pcbe, StreamThreshold[Index][IndexStrm], 0xa0a0a0a0, 0);
           }

           //timeout
           cbMMIOWriteReg32(pcbe, StreamTimeout[Index], 0x08080808, 0);

           // set HPD signal to high level active if this feature enabled
           if(1)//if(pcbe->FeatureSwitch.HPDActiveHighEnable)
           {
               cbMMIOWriteReg32(pcbe, HpdInvert[Index], 0, 0x7fffffff);
           }
           else
           {
               cbMMIOWriteReg32(pcbe, HpdInvert[Index], 0x80000000, 0x7fffffff);    
           }
       }

       //force high & enable new feature
       cbMMIOWriteReg(pcbe, CR_A5, 0x08, 0x00);

       //prefetch 4 line
       //HardWare bug:enable panel upscaler and enable SS at the same time,SS will display not correctly if prefetch 8 line


       cbBiosMMIOWriteReg(pcbe, CR_8F, 0x40, (CBIOS_U8)~0xc0, IGA1);
       cbBiosMMIOWriteReg(pcbe, CR_8F, 0x40, (CBIOS_U8)~0xc0, IGA2);
       cbBiosMMIOWriteReg(pcbe, CR_8F, 0x40, (CBIOS_U8)~0xc0, IGA3);
       if(CHIPID_E3K == pcbe->ChipID)
       {
           cbBiosMMIOWriteReg(pcbe, CR_8F, 0x40, (CBIOS_U8)~0xc0, IGA4);
       }
    }

    //Enable linear address access.
    cbMMIOWriteReg(pcbe,CR_C_A0,0x10, ~0x10);

    //Enable pt
    cbMMIOWriteReg(pcbe, CR_A5, 0x01, ~0x01);

    return CBIOS_OK;
}



